@@ -38,7 +38,7 @@
 			   (p)->type == para_Title ? -1 : 0 )
 
 typedef struct {
-    int number_at_all, just_numbers;
+    bool number_at_all, just_numbers;
     wchar_t *number_suffix;
 } sectlevel;
 
@@ -47,10 +47,11 @@ typedef struct {
     sectlevel achapter, *asect;
     int *contents_depths;	       /* 0=main, 1=chapter, 2=sect etc */
     int ncdepths;
-    int address_section, visible_version_id;
-    int leaf_contains_contents, leaf_smallest_contents;
-    int navlinks;
-    int rellinks;
+    bool address_section, visible_version_id;
+    bool leaf_contains_contents;
+    int leaf_smallest_contents;
+    bool navlinks;
+    bool rellinks;
     char *contents_filename;
     char *index_filename;
     char *template_filename;
@@ -136,7 +137,7 @@ typedef struct {
 typedef struct {
     htmlsect *section;
     char *fragment;
-    int generated, referenced;
+    bool generated, referenced;
 } htmlindexref;
 
 typedef struct {
@@ -149,6 +150,7 @@ typedef struct {
     void (*write)(void *write_ctx, const char *data, int len);
     int charset, restrict_charset;
     charset_state cstate;
+    errorstate *es;
     int ver;
     enum {
 	HO_NEUTRAL, HO_IN_TAG, HO_IN_EMPTY_TAG, HO_IN_TEXT
@@ -191,7 +193,7 @@ void ho_setup_file(htmloutput *ho, const char *filename)
         ho->write = ho_write_file;
         ho->write_ctx = fp;
     } else {
-        err_cantopenw(filename);
+        err_cantopenw(ho->es, filename);
         ho->write = ho_write_ignore; /* saves conditionalising rest of code */
     }
 }
@@ -266,10 +268,10 @@ void ho_finish(htmloutput *ho)
 #define HO_HACK_QUOTENOTHING 2
 #define HO_HACK_OMITQUOTES 4
 
-static int html_fragment_compare(void *av, void *bv)
+static int html_fragment_compare(const void *av, const void *bv, void *cmpctx)
 {
-    htmlfragment *a = (htmlfragment *)av;
-    htmlfragment *b = (htmlfragment *)bv;
+    const htmlfragment *a = (const htmlfragment *)av;
+    const htmlfragment *b = (const htmlfragment *)bv;
     int cmp;
 
     if ((cmp = strcmp(a->file->filename, b->file->filename)) != 0)
@@ -278,10 +280,10 @@ static int html_fragment_compare(void *av, void *bv)
 	return strcmp(a->fragment, b->fragment);
 }
 
-static int html_filename_compare(void *av, void *bv)
+static int html_filename_compare(const void *av, const void *bv, void *cmpctx)
 {
-    char *a = (char *)av;
-    char *b = (char *)bv;
+    const char *a = (const char *)av;
+    const char *b = (const char *)bv;
 
     return strcmp(a, b);
 }
@@ -313,7 +315,7 @@ static void html_text(htmloutput *ho, wchar_t const *str);
 static void html_text_nbsp(htmloutput *ho, wchar_t const *str);
 static void html_text_limit(htmloutput *ho, wchar_t const *str, int maxlen);
 static void html_text_limit_internal(htmloutput *ho, wchar_t const *text,
-				     int maxlen, int quote_quotes, int nbsp);
+				     int maxlen, bool quote_quotes, bool nbsp);
 static void html_nl(htmloutput *ho);
 static void html_raw(htmloutput *ho, char *text);
 static void html_raw_as_attr(htmloutput *ho, char *text);
@@ -333,9 +335,10 @@ static void html_contents_entry(htmloutput *ho, int depth, htmlsect *s,
 				htmlconfig *cfg);
 static void html_section_title(htmloutput *ho, htmlsect *s,
 			       htmlfile *thisfile, keywordlist *keywords,
-			       htmlconfig *cfg, int real);
+			       htmlconfig *cfg, bool real);
 
-static htmlconfig html_configure(paragraph *source, int chm_mode)
+static htmlconfig html_configure(paragraph *source, bool chm_mode,
+                                 errorstate *es)
 {
     htmlconfig ret;
     paragraph *p;
@@ -344,22 +347,22 @@ static htmlconfig html_configure(paragraph *source, int chm_mode)
      * Defaults.
      */
     ret.leaf_level = chm_mode ? -1 /* infinite */ : 2;
-    ret.achapter.just_numbers = FALSE;
-    ret.achapter.number_at_all = TRUE;
+    ret.achapter.just_numbers = false;
+    ret.achapter.number_at_all = true;
     ret.achapter.number_suffix = L": ";
     ret.nasect = 1;
     ret.asect = snewn(ret.nasect, sectlevel);
-    ret.asect[0].just_numbers = TRUE;
-    ret.asect[0].number_at_all = TRUE;
+    ret.asect[0].just_numbers = true;
+    ret.asect[0].number_at_all = true;
     ret.asect[0].number_suffix = L" ";
     ret.ncdepths = 0;
     ret.contents_depths = 0;
-    ret.visible_version_id = TRUE;
-    ret.address_section = chm_mode ? FALSE : TRUE;
-    ret.leaf_contains_contents = FALSE;
+    ret.visible_version_id = true;
+    ret.address_section = chm_mode ? false : true;
+    ret.leaf_contains_contents = false;
     ret.leaf_smallest_contents = 4;
-    ret.navlinks = chm_mode ? FALSE : TRUE;
-    ret.rellinks = TRUE;
+    ret.navlinks = chm_mode ? false : true;
+    ret.rellinks = true;
     ret.single_filename = dupstr("Manual.html");
     ret.contents_filename = dupstr("Contents.html");
     ret.index_filename = dupstr("IndexPage.html");
@@ -427,7 +430,7 @@ static htmlconfig html_configure(paragraph *source, int chm_mode)
     for (p = source; p; p = p->next) {
 	if (p->type == para_Config) {
 	    wchar_t *k = p->keyword;
-            int generic = FALSE;
+            bool generic = false;
 
             if (!chm_mode && !ustrnicmp(k, L"html-", 5)) {
                 k += 5;
@@ -440,15 +443,17 @@ static htmlconfig html_configure(paragraph *source, int chm_mode)
                 /* In this mode, only accept directives that don't
                  * vary completely between the HTML and CHM output
                  * types. */
-                generic = TRUE;
+                generic = true;
             } else {
                 continue;
             }
 
 	    if (!ustricmp(k, L"restrict-charset")) {
-		ret.restrict_charset = charset_from_ustr(&p->fpos, uadv(k));
+		ret.restrict_charset = charset_from_ustr(
+                    &p->fpos, uadv(k), es);
 	    } else if (!ustricmp(k, L"output-charset")) {
-		ret.output_charset = charset_from_ustr(&p->fpos, uadv(k));
+		ret.output_charset = charset_from_ustr(
+                    &p->fpos, uadv(k), es);
 	    } else if (!ustricmp(k, L"version")) {
 		wchar_t *vername = uadv(k);
 		static const struct {
@@ -468,7 +473,7 @@ static htmlconfig html_configure(paragraph *source, int chm_mode)
 			break;
 
 		if (i == lenof(versions))
-		    err_htmlver(&p->fpos, vername);
+		    err_htmlver(es, &p->fpos, vername);
 		else
 		    ret.htmlver = versions[i].ver;
 	    } else if (!ustricmp(k, L"single-filename")) {
@@ -501,7 +506,7 @@ static htmlconfig html_configure(paragraph *source, int chm_mode)
 			frag = adv(frag);
 		    }
 		} else
-		    err_cfginsufarg(&p->fpos, p->origkeyword, 1);
+		    err_cfginsufarg(es, &p->fpos, p->origkeyword, 1);
 	    } else if (!ustricmp(k, L"chapter-numeric")) {
 		ret.achapter.just_numbers = utob(uadv(k));
 	    } else if (!ustricmp(k, L"chapter-shownumber")) {
@@ -675,7 +680,7 @@ static htmlconfig html_configure(paragraph *source, int chm_mode)
                         chmname = diskname;
 
                     if (chmname[0] == '#' || chmname[0] == '$')
-                        err_chm_badname(&p->fpos, chmname);
+                        err_chm_badname(es, &p->fpos, chmname);
 
                     if (ret.nchmextrafiles >= ret.chmextrafilesize) {
                         ret.chmextrafilesize = ret.nchmextrafiles * 5 / 4 + 32;
@@ -706,7 +711,7 @@ static htmlconfig html_configure(paragraph *source, int chm_mode)
          * turn both off.
          */
         if (!ret.chm_filename ^ !ret.hhp_filename) {
-            err_chmnames();
+            err_chmnames(es);
             sfree(ret.chm_filename); ret.chm_filename = NULL;
             sfree(ret.hhp_filename); ret.hhp_filename = NULL;
         }
@@ -757,7 +762,7 @@ paragraph *chm_config_filename(char *filename)
 }
 
 static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
-                                indexdata *idx, int chm_mode)
+                                indexdata *idx, errorstate *es, bool chm_mode)
 {
     paragraph *p;
     htmlsect *topsect;
@@ -765,9 +770,9 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
     htmlfilelist files = { NULL, NULL, NULL, NULL, NULL, NULL };
     htmlsectlist sects = { NULL, NULL }, nonsects = { NULL, NULL };
     struct chm *chm = NULL;
-    int has_index, hhk_needed = FALSE;
+    bool has_index, hhk_needed = false;
 
-    conf = html_configure(sourceform, chm_mode);
+    conf = html_configure(sourceform, chm_mode, es);
 
     /*
      * We're going to make heavy use of paragraphs' private data
@@ -778,8 +783,8 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
     for (p = sourceform; p; p = p->next)
 	p->private_data = NULL;
 
-    files.frags = newtree234(html_fragment_compare);
-    files.files = newtree234(html_filename_compare);
+    files.frags = newtree234(html_fragment_compare, NULL);
+    files.files = newtree234(html_filename_compare, NULL);
 
     /*
      * Start by figuring out into which file each piece of the
@@ -979,7 +984,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 		    indextag *tag;
 		    int i;
 
-		    hr->referenced = hr->generated = FALSE;
+		    hr->referenced = hr->generated = false;
 		    hr->section = lastsect;
 		    {
 			char buf[40];
@@ -1037,7 +1042,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 
 	for (f = files.head; f; f = f->next) {
 	    htmloutput ho;
-	    int displaying;
+	    bool displaying;
 	    enum LISTTYPE { NOLIST, UL, OL, DL };
 	    enum ITEMTYPE { NOITEM, LI, DT, DD };
 	    struct stackelement {
@@ -1059,6 +1064,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 	    ho.charset = conf.output_charset;
 	    ho.restrict_charset = conf.restrict_charset;
 	    ho.cstate = charset_init_state;
+            ho.es = es;
 	    ho.ver = conf.htmlver;
 	    ho.state = HO_NEUTRAL;
 	    ho.contents_level = 0;
@@ -1310,7 +1316,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 			    html_fragment(&ho, sects.head->fragments[i]);
 		}
 
-		html_section_title(&ho, sects.head, f, keywords, &conf, TRUE);
+		html_section_title(&ho, sects.head, f, keywords, &conf, true);
 
 		element_close(&ho, "h1");
 	    }
@@ -1329,7 +1335,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 	    {
 		int ntoc = 0, tocsize = 0, tocstartidx = 0;
 		htmlsect **toc = NULL;
-		int leaf = TRUE;
+		bool leaf = true;
 
 		for (s = sects.head; s; s = s->next) {
 		    htmlsect *a, *ac;
@@ -1351,7 +1357,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 		    }
 
 		    if (s->file != f && a != NULL)
-			leaf = FALSE;
+			leaf = false;
 
 		    if (a) {
 			if (adepth <= a->contents_depth) {
@@ -1399,14 +1405,14 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 	     * Now go through the document and output some real
 	     * text.
 	     */
-	    displaying = FALSE;
+	    displaying = false;
 	    for (s = sects.head; s; s = s->next) {
 		if (s->file == f) {
 		    /*
 		     * This section belongs in this file.
 		     * Display it.
 		     */
-		    displaying = TRUE;
+		    displaying = true;
 		} else {
 		    /*
 		     * Doesn't belong in this file, but it may be
@@ -1417,7 +1423,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 		    htmlsect *a, *ac;
 		    int depth, adepth;
 
-		    displaying = FALSE;
+		    displaying = false;
 
 		    /*
 		     * Search up from this section until we find
@@ -1490,7 +1496,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 				    html_fragment(&ho, s->fragments[i]);
 			}
 
-			html_section_title(&ho, s, f, keywords, &conf, TRUE);
+			html_section_title(&ho, s, f, keywords, &conf, true);
 
 			element_close(&ho, htag);
 		    }
@@ -1722,7 +1728,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 
 				html_href(&ho, f, hr->section->file,
 					  hr->fragment);
-				hr->referenced = TRUE;
+				hr->referenced = true;
 				if (p && p->kwtext)
 				    html_words(&ho, p->kwtext, MARKUP|LINKS,
 					       f, keywords, &conf);
@@ -1755,7 +1761,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 		/*
 		 * Footer.
 		 */
-		int done_version_ids = FALSE;
+		bool done_version_ids = false;
 
 		if (conf.address_section)
 		    element_empty(&ho, "hr");
@@ -1764,7 +1770,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 		    html_raw(&ho, conf.body_end);
 
 		if (conf.address_section) {
-		    int started = FALSE;
+		    bool started = false;
 		    if (conf.htmlver == ISO_HTML) {
 			/*
 			 * The ISO-HTML validator complains if
@@ -1782,7 +1788,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 		    if (conf.addr_start) {
 			html_raw(&ho, conf.addr_start);
 			html_nl(&ho);
-			started = TRUE;
+			started = true;
 		    }
 		    if (conf.visible_version_id) {
 			for (p = sourceform; p; p = p->next)
@@ -1794,9 +1800,9 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 				html_words(&ho, p->words, NOTHING,
 					   f, keywords, &conf);
 				html_text(&ho, conf.post_versionid);
-				started = TRUE;
+				started = true;
 			    }
-			done_version_ids = TRUE;
+			done_version_ids = true;
 		    }
 		    if (conf.addr_end) {
 			if (started)
@@ -1814,12 +1820,12 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 		     * visible, I think we still have a duty to put
 		     * them in an HTML comment.
 		     */
-		    int started = FALSE;
+		    bool started = false;
 		    for (p = sourceform; p; p = p->next)
 			if (p->type == para_VersionID) {
 			    if (!started) {
 				html_raw(&ho, "<!-- version IDs:\n");
-				started = TRUE;
+				started = true;
 			    }
 			    html_words(&ho, p->words, NOTHING,
 				       f, keywords, &conf);
@@ -1844,7 +1850,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
      * if the index contains nothing.
      */
     if (chm_mode || conf.hhk_filename) {
-	int ok = FALSE;
+	bool ok = false;
 	int i;
 	indexentry *entry;
 
@@ -1852,13 +1858,13 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 	    htmlindex *hi = (htmlindex *)entry->backend_data;
 
 	    if (hi->nrefs > 0) {
-		ok = TRUE;	       /* found an index entry */
+		ok = true;	       /* found an index entry */
 		break;
 	    }
 	}
 
 	if (ok)
-	    hhk_needed = TRUE;
+	    hhk_needed = true;
     }
 
     /*
@@ -1879,6 +1885,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
             ho.charset = CS_CP1252; /* as far as I know, CHM is */
             ho.restrict_charset = CS_CP1252; /* hardwired to this charset */
             ho.cstate = charset_init_state;
+            ho.es = es;
             ho.ver = HTML_4;	       /* *shrug* */
             ho.state = HO_NEUTRAL;
             ho.contents_level = 0;
@@ -1887,8 +1894,9 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
             ho_setup_rdstringc(&ho, &rs);
 
             ho.hacklimit = 255;
-            html_words(&ho, topsect->title->words, NOTHING,
-                       NULL, keywords, &conf);
+            if (topsect->title)
+                html_words(&ho, topsect->title->words, NOTHING,
+                           NULL, keywords, &conf);
 
             rdaddc(&rs, '\0');
             chm_title(chm, rs.text);
@@ -1930,6 +1938,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
                 ho.charset = CS_CP1252;
                 ho.restrict_charset = CS_CP1252;
                 ho.cstate = charset_init_state;
+                ho.es = es;
                 ho.ver = HTML_4;	       /* *shrug* */
                 ho.state = HO_NEUTRAL;
                 ho.contents_level = 0;
@@ -1973,7 +1982,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 
                 fp = fopen(fname, "rb");
                 if (!fp) {
-                    err_cantopen(fname);
+                    err_cantopen(es, fname);
                     continue;
                 }
 
@@ -2003,6 +2012,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 	ho.charset = CS_CP1252;	       /* as far as I know, HHP files are */
 	ho.restrict_charset = CS_CP1252;   /* hardwired to this charset */
 	ho.cstate = charset_init_state;
+        ho.es = es;
 	ho.ver = HTML_4;	       /* *shrug* */
 	ho.state = HO_NEUTRAL;
 	ho.contents_level = 0;
@@ -2027,8 +2037,9 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
                   "Title=");
 
 	ho.hacklimit = 255;
-	html_words(&ho, topsect->title->words, NOTHING,
-		   NULL, keywords, &conf);
+        if (topsect->title)
+            html_words(&ho, topsect->title->words, NOTHING,
+                       NULL, keywords, &conf);
 
 	ho_string(&ho, "\n");
 
@@ -2101,6 +2112,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 	ho.charset = CS_CP1252;	       /* as far as I know, HHC files are */
 	ho.restrict_charset = CS_CP1252;   /* hardwired to this charset */
 	ho.cstate = charset_init_state;
+        ho.es = es;
 	ho.ver = HTML_4;	       /* *shrug* */
 	ho.state = HO_NEUTRAL;
 	ho.contents_level = 0;
@@ -2127,7 +2139,8 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 	    /*
 	     * For each HTML file, write out a contents entry.
 	     */
-	    int depth, leaf = TRUE;
+	    int depth;
+            bool leaf = true;
 
 	    /*
 	     * Determine the depth of this file in the contents
@@ -2160,7 +2173,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 		    if (leaf && s->file != f) {
 			for (a = s; a; a = a->parent)
 			    if (a->file == f) {
-				leaf = FALSE;
+				leaf = false;
 				break;
 			    }
 		    }
@@ -2218,6 +2231,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 	ho.charset = CS_CP1252;	       /* as far as I know, HHK files are */
 	ho.restrict_charset = CS_CP1252;   /* hardwired to this charset */
 	ho.cstate = charset_init_state;
+        ho.es = es;
 	ho.ver = HTML_4;	       /* *shrug* */
 	ho.state = HO_NEUTRAL;
 	ho.contents_level = 0;
@@ -2270,7 +2284,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 			hr->section->file->temp = 1;
 		    }
 
-		    hr->referenced = TRUE;
+		    hr->referenced = true;
 		}
 
 		ho_string(&ho, "</OBJECT>\n");
@@ -2301,7 +2315,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 
         fp = fopen(conf.chm_filename, "wb");
         if (!fp) {
-            err_cantopenw(conf.chm_filename);
+            err_cantopenw(es, conf.chm_filename);
         } else {
             data = chm_build(chm, &len);
             fwrite(data, 1, len, fp);
@@ -2324,7 +2338,7 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 	    if (w->type == word_IndexRef) {
 		htmlindexref *hr = (htmlindexref *)w->private_data;
 
-		assert(!hr->referenced == !hr->generated);
+		assert(hr->referenced == hr->generated);
 	    }
     }
 
@@ -2417,17 +2431,17 @@ static void html_backend_common(paragraph *sourceform, keywordlist *keywords,
 }
 
 void html_backend(paragraph *sourceform, keywordlist *keywords,
-                  indexdata *idx, void *unused)
+                  indexdata *idx, void *unused, errorstate *es)
 {
     IGNORE(unused);
-    html_backend_common(sourceform, keywords, idx, FALSE);
+    html_backend_common(sourceform, keywords, idx, es, false);
 }
 
 void chm_backend(paragraph *sourceform, keywordlist *keywords,
-                 indexdata *idx, void *unused)
+                 indexdata *idx, void *unused, errorstate *es)
 {
     IGNORE(unused);
-    html_backend_common(sourceform, keywords, idx, TRUE);
+    html_backend_common(sourceform, keywords, idx, es, true);
 }
 
 static void html_file_section(htmlconfig *cfg, htmlfilelist *files,
@@ -2621,7 +2635,7 @@ static void html_words(htmloutput *ho, word *words, int flags,
 	if (flags & INDEXENTS) {
 	    htmlindexref *hr = (htmlindexref *)w->private_data;
 	    html_fragment(ho, hr->fragment);
-	    hr->generated = TRUE;
+	    hr->generated = true;
 	}
 	break;
       case word_Normal:
@@ -2828,7 +2842,7 @@ static void element_attr_w(htmloutput *ho, char const *name,
     ho_string(ho, " ");
     ho_string(ho, name);
     ho_string(ho, "=\"");
-    html_text_limit_internal(ho, value, 0, TRUE, FALSE);
+    html_text_limit_internal(ho, value, 0, true, false);
     html_charset_cleanup(ho);
     ho_string(ho, "\"");
 }
@@ -2836,30 +2850,31 @@ static void element_attr_w(htmloutput *ho, char const *name,
 static void html_text(htmloutput *ho, wchar_t const *text)
 {
     return_mostly_to_neutral(ho);
-    html_text_limit_internal(ho, text, 0, FALSE, FALSE);
+    html_text_limit_internal(ho, text, 0, false, false);
 }
 
 static void html_text_nbsp(htmloutput *ho, wchar_t const *text)
 {
     return_mostly_to_neutral(ho);
-    html_text_limit_internal(ho, text, 0, FALSE, TRUE);
+    html_text_limit_internal(ho, text, 0, false, true);
 }
 
 static void html_text_limit(htmloutput *ho, wchar_t const *text, int maxlen)
 {
     return_mostly_to_neutral(ho);
-    html_text_limit_internal(ho, text, maxlen, FALSE, FALSE);
+    html_text_limit_internal(ho, text, maxlen, false, false);
 }
 
 static void html_text_limit_internal(htmloutput *ho, wchar_t const *text,
-				     int maxlen, int quote_quotes, int nbsp)
+				     int maxlen, bool quote_quotes, bool nbsp)
 {
     int textlen = ustrlen(text);
     char outbuf[256];
-    int bytes, err;
+    int bytes;
+    bool err;
 
     if (ho->hackflags & (HO_HACK_QUOTEQUOTES | HO_HACK_OMITQUOTES))
-	quote_quotes = TRUE;	       /* override the input value */
+	quote_quotes = true;	       /* override the input value */
 
     if (maxlen > 0 && textlen > maxlen)
 	textlen = maxlen;
@@ -2944,11 +2959,20 @@ static void html_href(htmloutput *ho, htmlfile *thisfile,
 	rdaddc(&rs, '#');
 	rdaddsc(&rs, targetfrag);
     }
+
+    /* If _neither_ of those conditions were true, we don't have a URL
+     * at all and will segfault when we pass url==NULL to element_attr.
+     *
+     * I think this can only occur as a knock-on effect from an input
+     * file error, but we still shouldn't crash, of course. */
+
     url = rs.text;
 
     element_open(ho, "a");
-    element_attr(ho, "href", url);
-    sfree(url);
+    if (url) {
+        element_attr(ho, "href", url);
+        sfree(url);
+    }
 }
 
 static void html_fragment(htmloutput *ho, char const *fragment)
@@ -3130,7 +3154,7 @@ static char *html_sanitise_filename(htmlfilelist *files, char *text)
 
 	p = NULL;
 
-	while (find234(files->files, text, NULL)) {
+	while (find234(files->files, text)) {
 	    if (!p) {
 		len = strlen(text);
 		p = text;
@@ -3183,14 +3207,14 @@ static void html_contents_entry(htmloutput *ho, int depth, htmlsect *s,
 
     element_open(ho, "li");
     html_href(ho, thisfile, s->file, s->fragments[0]);
-    html_section_title(ho, s, thisfile, keywords, cfg, FALSE);
+    html_section_title(ho, s, thisfile, keywords, cfg, false);
     element_close(ho, "a");
     /* <li> will be closed by a later invocation */
 }
 
 static void html_section_title(htmloutput *ho, htmlsect *s, htmlfile *thisfile,
 			       keywordlist *keywords, htmlconfig *cfg,
-			       int real)
+			       bool real)
 {
     if (s->title) {
 	sectlevel *sl;
