@@ -35,17 +35,17 @@ static void pdf_string_len(void (*add)(object *, char const *),
 static void objref(object *o, object *dest);
 static void objdest(object *o, page_data *p);
 
-static int is_std_font(char const *name);
+static bool is_std_font(char const *name);
 
 static void make_pages_node(object *node, object *parent, page_data *first,
 			    page_data *last, object *resources,
 			    object *mediabox);
 static int make_outline(object *parent, outline_element *start, int n,
-			int open);
+			bool open);
 static int pdf_versionid(FILE *fp, word *words);
 
 void pdf_backend(paragraph *sourceform, keywordlist *keywords,
-		 indexdata *idx, void *vdoc) {
+		 indexdata *idx, void *vdoc, errorstate *es) {
     document *doc = (document *)vdoc;
     int font_index;
     font_encoding *fe;
@@ -131,7 +131,7 @@ void pdf_backend(paragraph *sourceform, keywordlist *keywords,
 	char fname[40];
 	char buf[80];
 	int i, prev;
-	object *font, *fontdesc;
+	object *font, *fontdesc = NULL;
 	int flags;
 	font_info const *fi = fe->font->info;
 
@@ -301,8 +301,9 @@ void pdf_backend(paragraph *sourceform, keywordlist *keywords,
 	    objtext(cidfont, fe->font->info->name);
 	    objtext(cidfont, "\n/CIDSystemInfo<</Registry(Adobe)"
 		    "/Ordering(Identity)/Supplement 0>>\n");
-	    objtext(cidfont, "/FontDescriptor ");
-	    objref(cidfont, fontdesc);
+	    assert(fontdesc);  /* TrueType fonts are never standard */
+            objtext(cidfont, "/FontDescriptor ");
+            objref(cidfont, fontdesc);
 	    objtext(cidfont, "\n/W[0[");
 	    for (i = 0; i < (int)sfnt_nglyphs(fe->font->info->fontfile); i++) {
 		char buf[20];
@@ -328,7 +329,7 @@ void pdf_backend(paragraph *sourceform, keywordlist *keywords,
 		    objtext(font, buf);
 		}
 		objtext(font, i % 8 ? "/" : "\n/");
-		objtext(font, glyph_extern(fe->vector[i]));
+		objtext(font, glyph_extern(doc->psd, fe->vector[i]));
 		prev = i;
 	    }
 
@@ -369,12 +370,12 @@ void pdf_backend(paragraph *sourceform, keywordlist *keywords,
 		size_t len;
 		char *ffbuf;
 
-		pf_part1((font_info *)fi, &ffbuf, &len);
+		pf_part1((font_info *)fi, &ffbuf, &len, es);
 		objstream_len(fontfile, ffbuf, len);
 		sfree(ffbuf);
 		sprintf(buf, "<<\n/Length1 %lu\n", (unsigned long)len);
 		objtext(fontfile, buf);
-		pf_part2((font_info *)fi, &ffbuf, &len);
+		pf_part2((font_info *)fi, &ffbuf, &len, es);
 		objstream_len(fontfile, ffbuf, len);
 		sfree(ffbuf);
 		sprintf(buf, "/Length2 %lu\n", (unsigned long)len);
@@ -606,7 +607,7 @@ void pdf_backend(paragraph *sourceform, keywordlist *keywords,
 
 	objtext(outlines, "<<\n/Type /Outlines\n");
 	topcount = make_outline(outlines, doc->outline_elements,
-				doc->n_outline_elements, TRUE);
+				doc->n_outline_elements, true);
 	sprintf(buf, "/Count %d\n>>\n", topcount);
 	objtext(outlines, buf);
     }
@@ -672,7 +673,7 @@ void pdf_backend(paragraph *sourceform, keywordlist *keywords,
     else
 	fp = fopen(filename, "wb");
     if (!fp) {
-	err_cantopenw(filename);
+	err_cantopenw(es, filename);
 	return;
     }
 
@@ -784,12 +785,12 @@ static char const * const stdfonts[] = {
     "Symbol", "ZapfDingbats"
 };
 
-static int is_std_font(char const *name) {
+static bool is_std_font(char const *name) {
     unsigned i;
     for (i = 0; i < lenof(stdfonts); i++)
 	if (strcmp(name, stdfonts[i]) == 0)
-	    return TRUE;
-    return FALSE;
+	    return true;
+    return false;
 }
 
 static void make_pages_node(object *node, object *parent, page_data *first,
@@ -830,10 +831,12 @@ static void make_pages_node(object *node, object *parent, page_data *first,
 	for (i = 0; i < TREE_BRANCH; i++) {
 	    int number = (i+1) * count / TREE_BRANCH - i * count / TREE_BRANCH;
 	    thisfirst = page;
+            thislast = NULL;
 	    while (number--) {
 		thislast = page;
 		page = page->next;
 	    }
+            assert(thislast);
 
 	    if (thisfirst == thislast) {
 		objref(node, (object *)thisfirst->spare);
@@ -913,7 +916,7 @@ char *pdf_outline_convert(wchar_t *s, int *len) {
 }
 
 static int make_outline(object *parent, outline_element *items, int n,
-			int open)
+			bool open)
 {
     int level, totalcount = 0;
     outline_element *itemp;
@@ -965,7 +968,7 @@ static int make_outline(object *parent, outline_element *items, int n,
 
 	if (itemp > items) {
 	    char buf[80];
-	    int count = make_outline(curr, items, itemp - items, FALSE);
+	    int count = make_outline(curr, items, itemp - items, false);
 	    if (!open)
 		count = -count;
 	    else
@@ -1021,6 +1024,8 @@ static int pdf_versionid(FILE *fp, word *words)
 	  case word_Quote:
 	    text = dupstr("'");
 	    break;
+          default:
+            continue;
 	}
 
 	fputs(text, fp);
