@@ -22,7 +22,7 @@ paragraph *ps_config_filename(char *filename)
 }
 
 void ps_backend(paragraph *sourceform, keywordlist *keywords,
-		indexdata *idx, void *vdoc) {
+		indexdata *idx, void *vdoc, errorstate *es) {
     document *doc = (document *)vdoc;
     int font_index;
     font_encoding *fe;
@@ -53,7 +53,7 @@ void ps_backend(paragraph *sourceform, keywordlist *keywords,
     else
 	fp = fopen(filename, "w");
     if (!fp) {
-	err_cantopenw(filename);
+	err_cantopenw(es, filename);
 	return;
     }
 
@@ -205,7 +205,7 @@ void ps_backend(paragraph *sourceform, keywordlist *keywords,
 	    if (fe->font->info->filetype == TYPE1)
 		pf_writeps(fe->font->info, fp);
 	    else
-		sfnt_writeps(fe->font->info, fp);
+		sfnt_writeps(fe->font->info, fp, doc->psd, es);
 	    fprintf(fp, "%%%%EndResource\n");
 	} else {
 	    fprintf(fp, "%%%%IncludeResource: font %s\n",
@@ -229,7 +229,7 @@ void ps_backend(paragraph *sourceform, keywordlist *keywords,
 	ps_token(fp, &cc, "{1 index /FID ne {def} {pop pop} ifelse} forall\n");
 	ps_token(fp, &cc, "/Encoding [\n");
 	for (i = 0; i < 256; i++)
-	    ps_token(fp, &cc, "/%s", glyph_extern(fe->vector[i]));
+	    ps_token(fp, &cc, "/%s", glyph_extern(doc->psd, fe->vector[i]));
 	ps_token(fp, &cc, "] def\n");
 	ps_token(fp, &cc, "currentdict end\n");
 	ps_token(fp, &cc, "/fontname-%s exch definefont /%s exch def\n",
@@ -344,6 +344,8 @@ static void ps_comment(FILE *fp, char const *leader, word *words) {
 	  case word_Quote:
 	    text = dupstr("'");
 	    break;
+          default:
+            continue;
 	}
 
 	if (cc + strlen(text) > PS_MAXWIDTH)
