@@ -10,7 +10,7 @@
 typedef enum { LEFT, LEFTPLUS, CENTRE } alignment;
 typedef struct {
     alignment align;
-    int number_at_all, just_numbers;
+    bool number_at_all, just_numbers;
     wchar_t *underline;
     wchar_t *number_suffix;
 } alignstruct;
@@ -21,8 +21,8 @@ typedef struct {
     int width;
     alignstruct atitle, achapter, *asect;
     int nasect;
-    int include_version_id;
-    int indent_preambles;
+    bool include_version_id;
+    bool indent_preambles;
     int charset;
     word bullet;
     wchar_t *lquote, *rquote, *rule;
@@ -34,6 +34,7 @@ typedef struct {
     FILE *fp;
     int charset;
     charset_state state;
+    errorstate *es;
 } textfile;
 
 static void text_heading(textfile *, word *, word *, word *, alignstruct,
@@ -55,7 +56,7 @@ static alignment utoalign(wchar_t *p) {
     return LEFT;
 }
 
-static textconfig text_configure(paragraph *source) {
+static textconfig text_configure(paragraph *source, errorstate *es) {
     textconfig ret;
     paragraph *p;
     int n;
@@ -66,8 +67,8 @@ static textconfig text_configure(paragraph *source) {
     ret.bullet.next = NULL;
     ret.bullet.alt = NULL;
     ret.bullet.type = word_Normal;
-    ret.atitle.just_numbers = FALSE;   /* ignored */
-    ret.atitle.number_at_all = TRUE;   /* ignored */
+    ret.atitle.just_numbers = false;   /* ignored */
+    ret.atitle.number_at_all = true;   /* ignored */
 
     /*
      * Defaults.
@@ -80,19 +81,19 @@ static textconfig text_configure(paragraph *source) {
     ret.atitle.align = CENTRE;
     ret.atitle.underline = L"\x2550\0=\0\0";
     ret.achapter.align = LEFT;
-    ret.achapter.just_numbers = FALSE;
-    ret.achapter.number_at_all = TRUE;
+    ret.achapter.just_numbers = false;
+    ret.achapter.number_at_all = true;
     ret.achapter.number_suffix = L": ";
     ret.achapter.underline = L"\x203E\0-\0\0";
     ret.nasect = 1;
     ret.asect = snewn(ret.nasect, alignstruct);
     ret.asect[0].align = LEFTPLUS;
-    ret.asect[0].just_numbers = TRUE;
-    ret.asect[0].number_at_all = TRUE;
+    ret.asect[0].just_numbers = true;
+    ret.asect[0].number_at_all = true;
     ret.asect[0].number_suffix = L" ";
     ret.asect[0].underline = L"\0";
-    ret.include_version_id = TRUE;
-    ret.indent_preambles = FALSE;
+    ret.include_version_id = true;
+    ret.indent_preambles = false;
     ret.bullet.text = L"\x2022\0-\0\0";
     ret.rule = L"\x2500\0-\0\0";
     ret.filename = dupstr("output.txt");
@@ -131,7 +132,8 @@ static textconfig text_configure(paragraph *source) {
 	    if (!ustricmp(p->keyword, L"text-indent")) {
 		ret.indent = utoi(uadv(p->keyword));
 	    } else if (!ustricmp(p->keyword, L"text-charset")) {
-		ret.charset = charset_from_ustr(&p->fpos, uadv(p->keyword));
+		ret.charset = charset_from_ustr(
+                    &p->fpos, uadv(p->keyword), es);
 	    } else if (!ustricmp(p->keyword, L"text-filename")) {
 		sfree(ret.filename);
 		ret.filename = dupstr(adv(p->origkeyword));
@@ -318,7 +320,7 @@ paragraph *text_config_filename(char *filename)
 }
 
 void text_backend(paragraph *sourceform, keywordlist *keywords,
-		  indexdata *idx, void *unused) {
+		  indexdata *idx, void *unused, errorstate *es) {
     paragraph *p;
     textconfig conf;
     word *prefix, *body, *wp;
@@ -332,7 +334,7 @@ void text_backend(paragraph *sourceform, keywordlist *keywords,
     IGNORE(keywords);		       /* we don't happen to need this */
     IGNORE(idx);		       /* or this */
 
-    conf = text_configure(sourceform);
+    conf = text_configure(sourceform, es);
 
     /*
      * Open the output file.
@@ -342,10 +344,11 @@ void text_backend(paragraph *sourceform, keywordlist *keywords,
     else
 	tf.fp = fopen(conf.filename, "w");
     if (!tf.fp) {
-	err_cantopenw(conf.filename);
+	err_cantopenw(es, conf.filename);
 	return;
     }
     tf.charset = conf.charset;
+    tf.es = es;
     tf.state = charset_init_state;
 
     /* Do the title */
@@ -782,7 +785,7 @@ static void text_codepara(textfile *tf, word *text, int indent, int width) {
     for (; text; text = text->next) if (text->type == word_WeakCode) {
 	int wid = ustrwid(text->text, tf->charset);
 	if (wid > width)
-	    err_text_codeline(&text->fpos, wid, width);
+	    err_text_codeline(tf->es, &text->fpos, wid, width);
 	text_output_many(tf, indent, L' ');
 	text_output(tf, text->text);
 	text_output(tf, L"\n");
