@@ -7,6 +7,7 @@
 #define charset_charset_h
 
 #include <stddef.h>
+#include <stdbool.h>
 
 /*
  * Enumeration that lists all the multibyte or single-byte
@@ -79,6 +80,8 @@ typedef enum {
     CS_UTF16,
     CS_UTF16BE,
     CS_UTF16LE,
+    CS_UTF16BE_NO_BOM,
+    CS_UTF16LE_NO_BOM,
     CS_EUC_JP,
     CS_EUC_CN,
     CS_EUC_KR,
@@ -95,6 +98,10 @@ typedef enum {
     CS_BS4730,
     CS_DEC_GRAPHICS,
     CS_EUC_TW,
+    CS_ISO6937,
+    CS_ISO6937_EURO,
+    CS_ITS,
+    CS_SAIL,
     CS_LIMIT               /* dummy value indicating extent of enum */
 } charset_t;
 
@@ -109,6 +116,13 @@ typedef struct {
  */
 #define CHARSET_INIT_STATE { 0L, 0L }  /* a suitable initialiser */
 
+#if defined __cplusplus
+extern "C" {
+#if 0
+}
+#endif
+#endif
+
 /*
  * This external variable contains the same data, but is provided
  * for easy structure-copy assignment:
@@ -168,9 +182,9 @@ int charset_to_unicode(const char **input, int *inlen,
  * If `error' is non-NULL and a character is found which cannot be
  * expressed in the output charset, conversion will terminate at
  * that character (so `input' points to the offending character)
- * and `*error' will be set to TRUE; if `error' is non-NULL and no
+ * and `*error' will be set to true; if `error' is non-NULL and no
  * difficult characters are encountered, `*error' will be set to
- * FALSE. If `error' is NULL, difficult characters will simply be
+ * false. If `error' is NULL, difficult characters will simply be
  * ignored.
  * 
  * If `input' is NULL, this routine will output the necessary bytes
@@ -187,7 +201,7 @@ int charset_to_unicode(const char **input, int *inlen,
 
 int charset_from_unicode(const wchar_t **input, int *inlen,
 			 char *output, int outlen,
-			 int charset, charset_state *state, int *error);
+			 int charset, charset_state *state, bool *error);
 
 /*
  * Convert X11 encoding names to and from our charset identifiers.
@@ -234,16 +248,16 @@ int charset_from_emacsenc(const char *name);
 int charset_upgrade(int charset);
 
 /*
- * This function returns TRUE if the input charset is a vaguely
- * sensible superset of ASCII. That is, it returns FALSE for 7-bit
+ * This function returns true if the input charset is a vaguely
+ * sensible superset of ASCII. That is, it returns false for 7-bit
  * encoding formats such as HZ and UTF-7.
  */
-int charset_contains_ascii(int charset);
+bool charset_contains_ascii(int charset);
 
 /*
- * This function returns TRUE if the input charset is single-byte.
+ * This function returns true if the input charset is single-byte.
  */
-int charset_is_single_byte(int charset);
+bool charset_is_single_byte(int charset);
 
 /*
  * This function tries to deduce the CS_* identifier of the charset
@@ -285,6 +299,13 @@ int charset_from_html_prefix(const char *data, size_t len,
  *         if (charset_exists(cs))
  *             do_stuff_with(cs);
  */
-int charset_exists(int charset);
+bool charset_exists(int charset);
+
+#if defined __cplusplus
+#if 0
+{
+#endif
+}
+#endif
 
 #endif /* charset_charset_h */
