@@ -6,13 +6,13 @@
 #include "internal.h"
 
 #define ENUM_CHARSET(x) extern charset_spec const charset_##x;
-#include "enum.c"
+#include "enum.h"
 #undef ENUM_CHARSET
 
 static charset_spec const *const cs_table[] = {
 
 #define ENUM_CHARSET(x) &charset_##x,
-#include "enum.c"
+#include "enum.h"
 #undef ENUM_CHARSET
 
 };
@@ -28,12 +28,12 @@ charset_spec const *charset_find_spec(int charset)
     return NULL;
 }
 
-int charset_exists(int charset)
+bool charset_exists(int charset)
 {
     return charset_find_spec(charset) != NULL;
 }
 
-int charset_is_single_byte(int charset)
+bool charset_is_single_byte(int charset)
 {
     charset_spec const *spec = charset_find_spec(charset);
     return spec && spec->read == read_sbcs;
