@@ -86,7 +86,7 @@ static void decode_end(void *src, void *dest) {
 }
 #define d_end decode_end, 0, 0
 
-static void *decode(sfnt_decode *dec, void *src, void *end, void *dest) {
+static void *decode(const sfnt_decode *dec, void *src, void *end, void *dest) {
     while (dec->decoder != decode_end) {
 	if ((char *)src + dec->src_len > (char *)end) return NULL;
 	dec->decoder(src, (char *)dest + dec->dest_offset);
@@ -96,7 +96,7 @@ static void *decode(sfnt_decode *dec, void *src, void *end, void *dest) {
     return src;
 }
 
-static void *decoden(sfnt_decode *dec, void *src, void *end, void *dest,
+static void *decoden(const sfnt_decode *dec, void *src, void *end, void *dest,
 		     size_t size, size_t n) {
     while (n-- && src) {
 	src = decode(dec, src, end, dest);
@@ -106,9 +106,9 @@ static void *decoden(sfnt_decode *dec, void *src, void *end, void *dest,
 }
 
 /* Decoding specs for simple data types */
-sfnt_decode uint16_decode[] = { { d_uint16, 0 }, { d_end } };
-sfnt_decode int16_decode[]  = { { d_int16,  0 }, { d_end } };
-sfnt_decode uint32_decode[] = { { d_uint32, 0 }, { d_end } };
+const sfnt_decode uint16_decode[] = { { d_uint16, 0 }, { d_end } };
+const sfnt_decode int16_decode[]  = { { d_int16,  0 }, { d_end } };
+const sfnt_decode uint32_decode[] = { { d_uint32, 0 }, { d_end } };
 
 /* Offset subdirectory -- the start of the file */
 typedef struct offsubdir_Tag offsubdir;
@@ -116,7 +116,7 @@ struct offsubdir_Tag {
     unsigned scaler_type;
     unsigned numTables;
 };
-sfnt_decode offsubdir_decode[] = {
+const sfnt_decode offsubdir_decode[] = {
     { d_uint32,	offsetof(offsubdir, scaler_type) },
     { d_uint16, offsetof(offsubdir, numTables) },
     { d_skip(6) },
@@ -145,7 +145,7 @@ struct tabledir_Tag {
     unsigned offset;
     unsigned length;
 };
-sfnt_decode tabledir_decode[] = {
+const sfnt_decode tabledir_decode[] = {
     { d_uint32,	offsetof(tabledir, tag) },
     { d_uint32, offsetof(tabledir, checkSum) },
     { d_uint32, offsetof(tabledir, offset) },
@@ -160,7 +160,7 @@ struct t_OS_2_Tag {
     int sTypoAscender, sTypoDescender;
     int sxHeight, sCapHeight;
 };
-sfnt_decode t_OS_2_v0_decode[] = {
+const sfnt_decode t_OS_2_v0_decode[] = {
     { d_uint16, offsetof(t_OS_2, version) },
     { d_skip(66) }, /* xAvgCharWidth, usWeightClass, usWidthClass, fsType, */
     /* ySubscriptXSize, ySubscriptYSize, ySubscriptXOffset, */
@@ -170,7 +170,7 @@ sfnt_decode t_OS_2_v0_decode[] = {
     /* achVendID, fsSelection, usFirstCharIndex, usLastCharIndex */
     { d_end }
 };
-sfnt_decode t_OS_2_v1_decode[] = {
+const sfnt_decode t_OS_2_v1_decode[] = {
     { d_uint16, offsetof(t_OS_2, version) },
     { d_skip(66) }, /* xAvgCharWidth, usWeightClass, usWidthClass, fsType, */
     /* ySubscriptXSize, ySubscriptYSize, ySubscriptXOffset, */
@@ -184,7 +184,7 @@ sfnt_decode t_OS_2_v1_decode[] = {
     /* ulCodePageRange1, ulCodePageRange2 */
     { d_end }
 };
-sfnt_decode t_OS_2_v2_decode[] = {
+const sfnt_decode t_OS_2_v2_decode[] = {
     { d_uint16, offsetof(t_OS_2, version) },
     { d_skip(66) }, /* xAvgCharWidth, usWeightClass, usWidthClass, fsType, */
     /* ySubscriptXSize, ySubscriptYSize, ySubscriptXOffset, */
@@ -207,7 +207,7 @@ typedef struct t_cmap_Tag t_cmap;
 struct t_cmap_Tag {
     unsigned numTables;
 };
-sfnt_decode t_cmap_decode[] = {
+const sfnt_decode t_cmap_decode[] = {
     { d_skip(2) },
     { d_uint16, offsetof(t_cmap, numTables) },
     { d_end }
@@ -218,7 +218,7 @@ struct encodingrec_Tag {
     unsigned encodingID;
     unsigned offset;
 };
-sfnt_decode encodingrec_decode[] = {
+const sfnt_decode encodingrec_decode[] = {
     { d_uint16, offsetof(encodingrec, platformID) },
     { d_uint16, offsetof(encodingrec, encodingID) },
     { d_uint32, offsetof(encodingrec, offset) },
@@ -229,7 +229,7 @@ struct cmap4_Tag {
     unsigned length;
     unsigned segCountX2;
 };
-sfnt_decode cmap4_decode[] = {
+const sfnt_decode cmap4_decode[] = {
     { d_skip(2) }, /* format */
     { d_uint16, offsetof(cmap4, length) },
     { d_skip(2) }, /* language */
@@ -248,7 +248,7 @@ struct t_head_Tag {
     int xMin, yMin, xMax, yMax;
     int indexToLocFormat;
 };
-sfnt_decode t_head_decode[] = {
+const sfnt_decode t_head_decode[] = {
     { d_uint32, offsetof(t_head, version) },
     { d_uint32, offsetof(t_head, fontRevision) },
     { d_skip(8) }, /* checkSumAdjustment, magicNumber, flags */
@@ -275,7 +275,7 @@ struct t_hhea_Tag {
     int metricDataFormat;
     unsigned numOfLongHorMetrics;
 };
-sfnt_decode t_hhea_decode[] = {
+const sfnt_decode t_hhea_decode[] = {
     { d_uint32, offsetof(t_hhea, version) },
     { d_int16,	offsetof(t_hhea, ascent) },
     { d_int16,	offsetof(t_hhea, descent) },
@@ -287,7 +287,7 @@ sfnt_decode t_hhea_decode[] = {
 };
 
 /* Horizontal Metrics ('hmtx') table */
-sfnt_decode longhormetric_decode[] = {
+const sfnt_decode longhormetric_decode[] = {
     { d_uint16, 0 },
     { d_skip(2) },
     { d_end }
@@ -299,7 +299,7 @@ struct t_kern_Tag {
     unsigned version;
     unsigned nTables;
 };
-sfnt_decode t_kern_v0_decode[] = {
+const sfnt_decode t_kern_v0_decode[] = {
     { d_uint16, offsetof(t_kern, version) },
     { d_uint16, offsetof(t_kern, nTables) },
     { d_end }
@@ -310,7 +310,7 @@ struct kern_v0_subhdr_Tag {
     unsigned length;
     unsigned coverage;
 };
-sfnt_decode kern_v0_subhdr_decode[] = {
+const sfnt_decode kern_v0_subhdr_decode[] = {
     { d_uint16, offsetof(kern_v0_subhdr, version) },
     { d_uint16, offsetof(kern_v0_subhdr, length) },
     { d_uint16, offsetof(kern_v0_subhdr, coverage) },
@@ -322,7 +322,7 @@ sfnt_decode kern_v0_subhdr_decode[] = {
 #define KERN_V0_OVERRIDE	0x0008
 #define KERN_V0_FORMAT		0xff00
 #define KERN_V0_FORMAT_0	0x0000
-sfnt_decode t_kern_v1_decode[] = {
+const sfnt_decode t_kern_v1_decode[] = {
     { d_uint32, offsetof(t_kern, version) },
     { d_uint32, offsetof(t_kern, nTables) },
     { d_end }
@@ -332,7 +332,7 @@ struct kern_v1_subhdr_Tag {
     unsigned length;
     unsigned coverage;
 };
-sfnt_decode kern_v1_subhdr_decode[] = {
+const sfnt_decode kern_v1_subhdr_decode[] = {
     { d_uint32, offsetof(kern_v1_subhdr, length) },
     { d_uint16, offsetof(kern_v1_subhdr, coverage) },
     { d_skip(2) }, /* tupleIndex */
@@ -347,7 +347,7 @@ typedef struct kern_f0_Tag kern_f0;
 struct kern_f0_Tag {
     unsigned nPairs;
 };
-sfnt_decode kern_f0_decode[] = {
+const sfnt_decode kern_f0_decode[] = {
     { d_uint16, offsetof(kern_f0, nPairs) },
     { d_skip(6) }, /* searchRange, entrySelector, rangeShift */
     { d_end }
@@ -358,7 +358,7 @@ struct kern_f0_pair_Tag {
     unsigned right;
     int value;
 };
-sfnt_decode kern_f0_pair_decode[] = {
+const sfnt_decode kern_f0_pair_decode[] = {
     { d_uint16, offsetof(kern_f0_pair, left) },
     { d_uint16, offsetof(kern_f0_pair, right) },
     { d_int16, offsetof(kern_f0_pair, value) },
@@ -371,7 +371,7 @@ struct t_maxp_Tag {
     unsigned version;
     unsigned numGlyphs;
 };
-sfnt_decode t_maxp_decode[] = {
+const sfnt_decode t_maxp_decode[] = {
     { d_uint32, offsetof(t_maxp, version) },
     { d_uint16, offsetof(t_maxp, numGlyphs) },
     { d_end }
@@ -386,7 +386,7 @@ struct t_name_Tag {
     unsigned stringOffset;
     namerecord *nameRecord;
 };
-sfnt_decode t_name_decode[] = {
+const sfnt_decode t_name_decode[] = {
     { d_uint16,	offsetof(t_name, format) },
     { d_uint16,	offsetof(t_name, count) },
     { d_uint16,	offsetof(t_name, stringOffset) },
@@ -400,7 +400,7 @@ struct namerecord_Tag {
     unsigned length;
     unsigned offset;
 };
-sfnt_decode namerecord_decode[] = {
+const sfnt_decode namerecord_decode[] = {
     { d_uint16, offsetof(namerecord, platformID) },
     { d_uint16, offsetof(namerecord, encodingID) },
     { d_uint16, offsetof(namerecord, languageID) },
@@ -421,7 +421,7 @@ struct t_post_Tag {
     unsigned minMemType42;
     unsigned maxMemType42;
 };
-sfnt_decode t_post_decode[] = {
+const sfnt_decode t_post_decode[] = {
     { d_uint32, offsetof(t_post, format) },
     { d_int32,  offsetof(t_post, italicAngle) },
     { d_int16,	offsetof(t_post, underlinePosition) },
@@ -452,21 +452,21 @@ struct sfnt_Tag {
     unsigned minmem, maxmem;
 };
 
-static int sfnt_findtable(sfnt *sf, unsigned tag,
-			  void **startp, void **endp) {
+static bool sfnt_findtable(sfnt *sf, unsigned tag,
+                           void **startp, void **endp) {
     size_t i;
 
     for (i = 0; i < sf->osd.numTables; i++) {
 	if (sf->td[i].tag == tag) {
 	    *startp = (char *)sf->data + sf->td[i].offset;
 	    *endp = (char *)*startp + sf->td[i].length;
-	    return TRUE;
+	    return true;
 	}
     }
-    return FALSE;
+    return false;
 }
 
-static char *sfnt_psname(font_info *fi) {
+static char *sfnt_psname(font_info *fi, errorstate *es) {
     sfnt *sf = fi->fontfile;
     t_name name;
     void *ptr, *end;
@@ -475,7 +475,7 @@ static char *sfnt_psname(font_info *fi) {
     namerecord *nr;
 
     if (!sfnt_findtable(sf, TAG_name, &ptr, &end)) {
-	err_sfntnotable(&sf->pos, "name");
+	err_sfntnotable(es, &sf->pos, "name");
 	return NULL;
     }
     ptr = decode(t_name_decode, ptr, end, &name);
@@ -496,7 +496,7 @@ static char *sfnt_psname(font_info *fi) {
 	    }
 	}
     }
-    err_sfntnopsname(&sf->pos);
+    err_sfntnopsname(es, &sf->pos);
     return NULL;
 }
 
@@ -520,11 +520,11 @@ static int glyphsbyname_cmp_search(void const *a, void const *b) {
 }
 
 /* Generate an name for a glyph that doesn't have one. */
-static glyph genglyph(unsigned idx) {
-    char buf[11];
-    if (idx == 0) return glyph_intern(".notdef");
+static glyph genglyph(psdata *psd, unsigned idx) {
+    char buf[64];
+    if (idx == 0) return glyph_intern(psd, ".notdef");
     sprintf(buf, "glyph%u", idx);
-    return glyph_intern(buf);
+    return glyph_intern(psd, buf);
 }
 
 /*
@@ -533,7 +533,7 @@ static glyph genglyph(unsigned idx) {
  * TODO: cope better with duplicated glyph names (usually .notdef)
  * TODO: when presented with format 3.0, try to use 'CFF' if present.
  */
-static void sfnt_mapglyphs(font_info *fi) {
+static void sfnt_mapglyphs(font_info *fi, psdata *psd, errorstate *es) {
     sfnt *sf = fi->fontfile;
     t_post post;
     void *ptr, *end;
@@ -546,7 +546,7 @@ static void sfnt_mapglyphs(font_info *fi) {
     if (sfnt_findtable(sf, TAG_post, &ptr, &end)) {
 	ptr = decode(t_post_decode, ptr, end, &post);
 	if (ptr == NULL) {
-	    err_sfntbadtable(&sf->pos, "post");
+	    err_sfntbadtable(es, &sf->pos, "post");
 	    goto noglyphs;
 	}
     
@@ -556,19 +556,19 @@ static void sfnt_mapglyphs(font_info *fi) {
 	switch (post.format) {
 	  case 0x00010000:
 	    if (sf->nglyphs != 258) {
-		err_sfntbadtable(&sf->pos, "post");
+		err_sfntbadtable(es, &sf->pos, "post");
 		break;
 	    }
 	    sf->glyphsbyindex = (glyph *)tt_std_glyphs;
 	    break;
 	  case 0x00020000:
 	    if ((char *)ptr + 2 > (char *)end) {
-		err_sfntbadtable(&sf->pos, "post");
+		err_sfntbadtable(es, &sf->pos, "post");
 		break;
 	    }
 	    ptr = (char *)ptr + 2;
 	    if ((char *)ptr + 2*sf->nglyphs > (char *)end) {
-		err_sfntbadtable(&sf->pos, "post");
+		err_sfntbadtable(es, &sf->pos, "post");
 		break;
 	    }
 	    nextras = 0;
@@ -584,7 +584,7 @@ static void sfnt_mapglyphs(font_info *fi) {
 		memcpy(tmp, sptr + 1, *sptr);
 		tmp[*sptr] = 0;
 		assert(i < nextras);
-		extraglyphs[i++] = glyph_intern(tmp);
+		extraglyphs[i++] = glyph_intern(psd, tmp);
 	    }
 	    sf->glyphsbyindex = snewn(sf->nglyphs, glyph);
 	    for (i = 0; i < sf->nglyphs; i++) {
@@ -594,8 +594,8 @@ static void sfnt_mapglyphs(font_info *fi) {
 		else if (g < 258 + nextras)
 		    sf->glyphsbyindex[i] = extraglyphs[g - 258];
 		else {
-		    err_sfntbadtable(&sf->pos, "post");
-		    sf->glyphsbyindex[i] = genglyph(i);
+		    err_sfntbadtable(es, &sf->pos, "post");
+		    sf->glyphsbyindex[i] = genglyph(psd, i);
 		}
 	    }
 	    sfree(extraglyphs);
@@ -603,7 +603,7 @@ static void sfnt_mapglyphs(font_info *fi) {
 	  case 0x00030000:
 	    break;
 	  default:
-	    err_sfnttablevers(&sf->pos, "post");
+	    err_sfnttablevers(es, &sf->pos, "post");
 	    break;
 	}
     }
@@ -611,7 +611,7 @@ static void sfnt_mapglyphs(font_info *fi) {
     if (!sf->glyphsbyindex) {
 	sf->glyphsbyindex = snewn(sf->nglyphs, glyph);
 	for (i = 0; i < sf->nglyphs; i++)
-	    sf->glyphsbyindex[i] = genglyph(i);
+	    sf->glyphsbyindex[i] = genglyph(psd, i);
     }
     /* Construct glyphsbyname */
     sf->glyphsbyname = snewn(sf->nglyphs, unsigned short);
@@ -633,7 +633,7 @@ static void sfnt_mapglyphs(font_info *fi) {
     suflen = 4;
     for (i = 0; i < sf->nglyphs; i++) {
 	char const *p;
-	p = strrchr(glyph_extern(sfnt_indextoglyph(sf, i)), '.');
+	p = strrchr(glyph_extern(psd, sfnt_indextoglyph(sf, i)), '.');
 	if (p && !(p+1)[strspn(p+1, "0123456789")] && strlen(p+1) > suflen)
 	    suflen = strlen(p+1);
     }
@@ -643,12 +643,12 @@ static void sfnt_mapglyphs(font_info *fi) {
 	if (prev == (this = sfnt_indextoglyph(sf, sf->glyphsbyname[i]))) {
 	    char const *basename;
 	    char *buf;
-	    basename = glyph_extern(this);
+	    basename = glyph_extern(psd, this);
 	    buf = snewn(strlen(basename) + 2 + suflen, char);
 	    strcpy(buf, basename);
 	    sprintf(buf + strlen(basename), ".%0*hu", suflen,
 		    sf->glyphsbyname[i]);
-	    sf->glyphsbyindex[sf->glyphsbyname[i]] = glyph_intern(buf);
+	    sf->glyphsbyindex[sf->glyphsbyname[i]] = glyph_intern(psd, buf);
 	    sfree(buf);
 	}
 	prev = this;
@@ -676,7 +676,7 @@ unsigned sfnt_glyphtoindex(sfnt *sf, glyph g) {
 /*
  * Get data from 'hhea', 'hmtx', and 'OS/2' tables
  */
-void sfnt_getmetrics(font_info *fi) {
+void sfnt_getmetrics(font_info *fi, errorstate *es) {
     sfnt *sf = fi->fontfile;
     t_hhea hhea;
     t_OS_2 OS_2;
@@ -690,31 +690,31 @@ void sfnt_getmetrics(font_info *fi) {
     fi->fontbbox[2] = sf->head.xMax * FUNITS_PER_PT /  sf->head.unitsPerEm;
     fi->fontbbox[3] = sf->head.yMax * FUNITS_PER_PT /  sf->head.unitsPerEm;
     if (!sfnt_findtable(sf, TAG_hhea, &ptr, &end)) {
-	err_sfntnotable(&sf->pos, "hhea");
+	err_sfntnotable(es, &sf->pos, "hhea");
 	return;
     }
     if (decode(t_hhea_decode, ptr, end, &hhea) == NULL) {
-	err_sfntbadtable(&sf->pos, "hhea");
+	err_sfntbadtable(es, &sf->pos, "hhea");
 	return;
     }
     if ((hhea.version & 0xffff0000) != 0x00010000) {
-	err_sfnttablevers(&sf->pos, "hhea");
+	err_sfnttablevers(es, &sf->pos, "hhea");
 	return;
     }
     fi->ascent = hhea.ascent;
     fi->descent = hhea.descent;
     if (hhea.metricDataFormat != 0) {
-	err_sfnttablevers(&sf->pos, "hmtx");
+	err_sfnttablevers(es, &sf->pos, "hmtx");
 	return;
     }
     if (!sfnt_findtable(sf, TAG_hmtx, &ptr, &end)) {
-	err_sfntnotable(&sf->pos, "hmtx");
+	err_sfntnotable(es, &sf->pos, "hmtx");
 	return;
     }
     hmtx = snewn(hhea.numOfLongHorMetrics, unsigned);
     if (decoden(longhormetric_decode, ptr, end, hmtx, sizeof(*hmtx),
 		hhea.numOfLongHorMetrics) == NULL) {
-	err_sfntbadtable(&sf->pos, "hmtx");
+	err_sfntbadtable(es, &sf->pos, "hmtx");
 	return;
     }
     for (i = 0; i < sf->nglyphs; i++) {
@@ -743,7 +743,7 @@ void sfnt_getmetrics(font_info *fi) {
     fi->descent = OS_2.sTypoDescender * FUNITS_PER_PT / sf->head.unitsPerEm;
     return;
   bados2:
-    err_sfntbadtable(&sf->pos, "OS/2");
+    err_sfntbadtable(es, &sf->pos, "OS/2");
 }
 
 /*
@@ -755,7 +755,7 @@ void sfnt_getmetrics(font_info *fi) {
  * pairs for horizontal kerning of horizontal text, and ignores
  * everything else.
  */
-static void sfnt_getkern(font_info *fi) {
+static void sfnt_getkern(font_info *fi, errorstate *es) {
     sfnt *sf = fi->fontfile;
     t_kern kern;
     unsigned version, i, j;
@@ -813,7 +813,7 @@ static void sfnt_getkern(font_info *fi) {
     }
     return;
   bad:
-    err_sfntbadtable(&sf->pos, "kern");
+    err_sfntbadtable(es, &sf->pos, "kern");
     return;
 }
 
@@ -825,7 +825,7 @@ static void sfnt_getkern(font_info *fi) {
  * Unicode 1.1 with precomposed Hangul syllables.  We only handle
  * format 4 of this table, since that seems to be the only one in use.
  */
-void sfnt_getmap(font_info *fi) {
+static void sfnt_getmap(font_info *fi, errorstate *es) {
     sfnt *sf = fi->fontfile;
     t_cmap cmap;
     encodingrec *esd;
@@ -837,7 +837,8 @@ void sfnt_getmap(font_info *fi) {
     for (i = 0; i < lenof(fi->bmp); i++)
 	    fi->bmp[i] = 0xFFFF;
     if (!sfnt_findtable(sf, TAG_cmap, &ptr, &end)) {
-	err_sfntnotable(&sf->pos, "cmap");
+	err_sfntnotable(es, &sf->pos, "cmap");
+        return;
     }
     base = ptr;
     ptr = decode(t_cmap_decode, ptr, end, &cmap);
@@ -886,7 +887,7 @@ void sfnt_getmap(font_info *fi) {
 			    idx = (k + idDelta[j]) & 0xffff;
 			    if (idx != 0) {
 				if (idx > sf->nglyphs)  {
-				    err_sfntbadglyph(&sf->pos, k);
+				    err_sfntbadglyph(es, &sf->pos, k);
 				    continue;
 				}
 				fi->bmp[k] = sfnt_indextoglyph(sf, idx);
@@ -897,14 +898,14 @@ void sfnt_getmap(font_info *fi) {
 			for (k = startCode[j]; k <= endCode[j]; k++) {
 			    if (startidx + k - startCode[j] >=
 				nglyphindex) {
-				err_sfntbadglyph(&sf->pos, k);
+				err_sfntbadglyph(es, &sf->pos, k);
 				continue;
 			    }
 			    idx = glyphIndexArray[startidx + k - startCode[j]];
 			    if (idx != 0) {
 				idx = (idx + idDelta[j]) & 0xffff;
 				if (idx > sf->nglyphs) {
-				    err_sfntbadglyph(&sf->pos, k);
+				    err_sfntbadglyph(es, &sf->pos, k);
 				    continue;
 				}
 				fi->bmp[k] = sfnt_indextoglyph(sf, idx);
@@ -917,13 +918,13 @@ void sfnt_getmap(font_info *fi) {
 	    }
 	}
     }
-    err_sfntnounicmap(&sf->pos);
+    err_sfntnounicmap(es, &sf->pos);
     return;
   bad:
-    err_sfntbadtable(&sf->pos, "cmap");
+    err_sfntbadtable(es, &sf->pos, "cmap");
 }
 
-void read_sfnt_file(input *in) {
+void read_sfnt_file(input *in, psdata *psd) {
     sfnt *sf = snew(sfnt);
     size_t off = 0, got;
     FILE *fp = in->currfp;
@@ -932,9 +933,9 @@ void read_sfnt_file(input *in) {
     t_maxp maxp;
 
     fi->name = NULL;
-    fi->widths = newtree234(width_cmp);
-    fi->kerns = newtree234(kern_cmp);
-    fi->ligs = newtree234(lig_cmp);
+    fi->widths = newtree234(width_cmp, NULL);
+    fi->kerns = newtree234(kern_cmp, NULL);
+    fi->ligs = newtree234(lig_cmp, NULL);
     fi->fontbbox[0] = fi->fontbbox[1] = fi->fontbbox[2] = fi->fontbbox[3] = 0;
     fi->capheight = fi->xheight = fi->ascent = fi->descent = 0;
     fi->stemh = fi->stemv = fi->italicangle = 0;
@@ -959,49 +960,49 @@ void read_sfnt_file(input *in) {
     sf->nglyphs = 0;
     ptr = decode(offsubdir_decode, sf->data, sf->end, &sf->osd);
     if (ptr == NULL) {
-	err_sfntbadhdr(&sf->pos);
+	err_sfntbadhdr(in->es, &sf->pos);
 	return;
     }
     sf->td = snewn(sf->osd.numTables, tabledir);
     ptr = decoden(tabledir_decode, ptr, sf->end, sf->td, sizeof(*sf->td),
 		  sf->osd.numTables);
     if (ptr == NULL) {
-	err_sfntbadhdr(&sf->pos);
+	err_sfntbadhdr(in->es, &sf->pos);
 	return;
     }	
     if (!sfnt_findtable(sf, TAG_head, &ptr, &end)) {
-	err_sfntnotable(&sf->pos, "head");
+	err_sfntnotable(in->es, &sf->pos, "head");
 	return;
     }
     if (decode(t_head_decode, ptr, end, &sf->head) == NULL) {
-	err_sfntbadtable(&sf->pos, "head");
+	err_sfntbadtable(in->es, &sf->pos, "head");
 	return;
     }
     if ((sf->head.version & 0xffff0000) != 0x00010000) {
-	err_sfnttablevers(&sf->pos, "head");
+	err_sfnttablevers(in->es, &sf->pos, "head");
 	return;
     }
     if (!sfnt_findtable(sf, TAG_maxp, &ptr, &end)) {
-	err_sfntnotable(&sf->pos, "maxp");
+	err_sfntnotable(in->es, &sf->pos, "maxp");
 	return;
     }
     if (decode(t_maxp_decode, ptr, end, &maxp) == NULL) {
-	err_sfntbadtable(&sf->pos, "maxp");
+	err_sfntbadtable(in->es, &sf->pos, "maxp");
 	return;
     }
     if (maxp.version < 0x00005000 || maxp.version > 0x0001ffff) {
-	err_sfnttablevers(&sf->pos, "maxp");
+	err_sfnttablevers(in->es, &sf->pos, "maxp");
 	return;
     }
     sf->nglyphs = maxp.numGlyphs;
-    fi->name = sfnt_psname(fi);
+    fi->name = sfnt_psname(fi, in->es);
     if (fi->name == NULL) return;
-    sfnt_mapglyphs(fi);
-    sfnt_getmetrics(fi);
-    sfnt_getkern(fi);
-    sfnt_getmap(fi);
-    fi->next = all_fonts;
-    all_fonts = fi;
+    sfnt_mapglyphs(fi, psd, in->es);
+    sfnt_getmetrics(fi, in->es);
+    sfnt_getkern(fi, in->es);
+    sfnt_getmap(fi, in->es);
+    fi->next = psd->all_fonts;
+    psd->all_fonts = fi;
 }
 
 static int sizecmp(const void *a, const void *b) {
@@ -1016,7 +1017,7 @@ static int sizecmp(const void *a, const void *b) {
  * <http://partners.adobe.com/public/developer/en/font/5012.Type42_Spec.pdf>
  */
 
-void sfnt_writeps(font_info const *fi, FILE *ofp) {
+void sfnt_writeps(font_info const *fi, FILE *ofp, psdata *psd, errorstate *es) {
     unsigned i, j, lastbreak;
     sfnt *sf = fi->fontfile;
     size_t *breaks, glyfoff, glyflen;
@@ -1053,7 +1054,7 @@ void sfnt_writeps(font_info const *fi, FILE *ofp) {
     fprintf(ofp, "0 1 %u{currentfile token pop exch def}bind for\n",
 	sf->nglyphs - 1);
     for (i = 0; i < sf->nglyphs; i++)
-	ps_token(ofp, &cc, "/%s", glyph_extern(sfnt_indextoglyph(sf, i)));
+	ps_token(ofp, &cc, "/%s", glyph_extern(psd, sfnt_indextoglyph(sf, i)));
     fprintf(ofp, "\nend readonly def\n");
     fprintf(ofp, "/sfnts [<");
     breaks = snewn(sf->osd.numTables + sf->nglyphs, size_t);
@@ -1061,13 +1062,13 @@ void sfnt_writeps(font_info const *fi, FILE *ofp) {
 	breaks[i] = sf->td[i].offset;
     }
     if (!sfnt_findtable(sf, TAG_glyf, &glyfptr, &glyfend)) {
-	err_sfntnotable(&sf->pos, "glyf");
+	err_sfntnotable(es, &sf->pos, "glyf");
 	return;
     }
     glyfoff = (char *)glyfptr - (char *)sf->data;
     glyflen = (char *)glyfend - (char *)glyfptr;
     if (!sfnt_findtable(sf, TAG_loca, &locaptr, &locaend)) {
-	err_sfntnotable(&sf->pos, "loca");
+	err_sfntnotable(es, &sf->pos, "loca");
 	return;
     }
     loca = snewn(sf->nglyphs, unsigned);
@@ -1100,7 +1101,7 @@ void sfnt_writeps(font_info const *fi, FILE *ofp) {
     fprintf(ofp, "end /%s exch definefont\n", fi->name);
     return;
   badloca:
-    err_sfntbadtable(&sf->pos, "loca");
+    err_sfntbadtable(es, &sf->pos, "loca");
 }
 
 void sfnt_data(font_info *fi, char **bufp, size_t *lenp) {
