@@ -6,13 +6,13 @@
 #include <stdlib.h>
 #include "halibut.h"
 
-static int compare_tags(void *av, void *bv);
-static int compare_entries(void *av, void *bv);
+static int compare_tags(const void *av, const void *bv, void *cmpctx);
+static int compare_entries(const void *av, const void *bv, void *cmpctx);
 
 indexdata *make_index(void) {
     indexdata *ret = snew(indexdata);
-    ret->tags = newtree234(compare_tags);
-    ret->entries = newtree234(compare_entries);
+    ret->tags = newtree234(compare_tags, NULL);
+    ret->entries = newtree234(compare_entries, NULL);
     return ret;
 }
 
@@ -27,18 +27,18 @@ static indextag *make_indextag(void) {
     return ret;
 }
 
-static int compare_tags(void *av, void *bv) {
-    indextag *a = (indextag *)av, *b = (indextag *)bv;
+static int compare_tags(const void *av, const void *bv, void *cmpctx) {
+    const indextag *a = (const indextag *)av, *b = (const indextag *)bv;
     return ustricmp(a->name, b->name);
 }
 
-static int compare_to_find_tag(void *av, void *bv) {
-    wchar_t *a = (wchar_t *)av;
-    indextag *b = (indextag *)bv;
+static int compare_to_find_tag(const void *av, const void *bv, void *cmpctx) {
+    const wchar_t *a = (const wchar_t *)av;
+    const indextag *b = (const indextag *)bv;
     return ustricmp(a, b->name);
 }
 
-static int compare_entries(void *av, void *bv) {
+static int compare_entries(const void *av, const void *bv, void *cmpctx) {
     indexentry *a = (indexentry *)av, *b = (indexentry *)bv;
     return compare_wordlists(a->text, b->text);    
 }
@@ -47,7 +47,7 @@ static int compare_entries(void *av, void *bv) {
  * Back-end utility: find the indextag with a given name.
  */
 indextag *index_findtag(indexdata *idx, wchar_t *name) {
-    return find234(idx->tags, name, compare_to_find_tag);
+    return findcmp234(idx->tags, name, compare_to_find_tag, NULL);
 }
 
 /*
@@ -58,8 +58,8 @@ indextag *index_findtag(indexdata *idx, wchar_t *name) {
  * Guarantee on calling sequence: all implicit merges are given
  * before the explicit ones.
  */
-void index_merge(indexdata *idx, int is_explicit, wchar_t *tags, word *text,
-		 filepos *fpos) {
+void index_merge(indexdata *idx, bool is_explicit, wchar_t *tags, word *text,
+		 filepos *fpos, errorstate *es) {
     indextag *t, *existing;
 
     /*
@@ -99,7 +99,7 @@ void index_merge(indexdata *idx, int is_explicit, wchar_t *tags, word *text,
 	     * warn (and drop it, since it won't be referenced).
 	     */
 	    if (is_explicit) {
-		err_nosuchidxtag(fpos, tags);
+		err_nosuchidxtag(es, fpos, tags);
 		continue;
 	    }
 
@@ -123,7 +123,7 @@ void index_merge(indexdata *idx, int is_explicit, wchar_t *tags, word *text,
 		 * see if the cases match.
 		 */
 		if (ustrcmp(t->name, existing->name)) {
-		    err_indexcase(fpos, t->name,
+		    err_indexcase(es, fpos, t->name,
 			  &existing->implicit_fpos, existing->name);
 		}
 
@@ -214,7 +214,7 @@ void cleanup_index(indexdata *i) {
 }
 
 static void dbg_prtwordlist(int level, word *w);
-static void dbg_prtmerge(int is_explicit, wchar_t *tag, word *text);
+static void dbg_prtmerge(bool is_explicit, wchar_t *tag, word *text);
 
 void index_debug(indexdata *i) {
     indextag *t;
@@ -240,7 +240,7 @@ void index_debug(indexdata *i) {
     }
 }
 
-static void dbg_prtmerge(int is_explicit, wchar_t *tag, word *text) {
+static void dbg_prtmerge(bool is_explicit, wchar_t *tag, word *text) {
     printf("\\IM: %splicit: \"", is_explicit ? "ex" : "im");
     for (; *tag; tag++)
 	putchar(*tag);
