@@ -39,26 +39,26 @@ struct macrostack_Tag {
     int ptr, npushback;
     filepos pos;
 };
-static int macrocmp(void *av, void *bv) {
+static int macrocmp(const void *av, const void *bv, void *cmpctx) {
     macro *a = (macro *)av, *b = (macro *)bv;
     return ustrcmp(a->name, b->name);
 }
 static void macrodef(tree234 *macros, wchar_t *name, wchar_t *text,
-		     filepos fpos) {
+		     filepos fpos, errorstate *es) {
     macro *m = snew(macro);
     m->name = name;
     m->text = text;
     if (add234(macros, m) != m) {
-	err_macroexists(&fpos, name);
+	err_macroexists(es, &fpos, name);
 	sfree(name);
 	sfree(text);
     }
 }
-static int macrolookup(tree234 *macros, input *in, wchar_t *name,
-		       filepos *pos) {
+static bool macrolookup(tree234 *macros, input *in, wchar_t *name,
+                        filepos *pos) {
     macro m, *gotit;
     m.name = name;
-    gotit = find234(macros, &m, NULL);
+    gotit = find234(macros, &m);
     if (gotit) {
 	macrostack *expansion = snew(macrostack);
 	expansion->next = in->stack;
@@ -67,9 +67,9 @@ static int macrolookup(tree234 *macros, input *in, wchar_t *name,
 	expansion->ptr = 0;
 	expansion->npushback = in->npushback;
 	in->stack = expansion;
-	return TRUE;
+	return true;
     } else
-	return FALSE;
+	return false;
 }
 static void macrocleanup(tree234 *macros) {
     int ti;
@@ -86,7 +86,8 @@ static void input_configure(input *in, paragraph *cfg) {
     assert(cfg->type == para_Config);
 
     if (!ustricmp(cfg->keyword, L"input-charset")) {
-	in->charset = charset_from_ustr(&cfg->fpos, uadv(cfg->keyword));
+	in->charset = charset_from_ustr(&cfg->fpos, uadv(cfg->keyword),
+                                        in->es);
     }
 }
 
@@ -169,11 +170,21 @@ static int get(input *in, filepos *pos, rdstringc *rsc) {
 					     NULL, 0);
 		assert(p == buf+1 && inlen == 0);
 
+                for (int i = 0; i < in->nwc; i++) {
+                    if (in->wc[i] == 0) {
+                        /* The zero Unicode character is never legal */
+                        err_zerochar(in->es, pos);
+                        return EOF;
+                    }
+                }
+
 		in->wcpos = 0;
 	    }
 	}
 
-	return in->wc[in->wcpos++];
+        wchar_t wc = in->wc[in->wcpos++];
+
+	return wc;
 
     } else
 	return EOF;
@@ -346,11 +357,13 @@ static void match_kw(token *tok) {
 	/* We expect hex characters thereafter. */
 	wchar_t *p = tok->text+1;
 	int n = 0;
+        bool seen_a_char = false;
 	while (*p && ishex(*p)) {
+            seen_a_char = true;
 	    n = 16 * n + fromhex(*p);
 	    p++;
 	}
-	if (!*p) {
+	if (!*p && seen_a_char) {
 	    tok->cmd = c_u;
 	    tok->aux = n;
 	    return;
@@ -478,10 +491,10 @@ token get_token(input *in) {
 	 * things other than whitespace, backslash, braces and
 	 * hyphen. A hyphen terminates the word but is returned as
 	 * part of it; everything else is pushed back for the next
-	 * token. The `aux' field contains TRUE if the word ends in
+	 * token. The `aux' field contains true if the word ends in
 	 * a hyphen.
 	 */
-	ret.aux = FALSE;	       /* assumed for now */
+	ret.aux = false;	       /* assumed for now */
 	prevpos = 0;
 	while (1) {
 	    if (iswhite(c) || c=='{' || c=='}' || c=='\\' || c==EOF) {
@@ -492,7 +505,7 @@ token get_token(input *in) {
 		rdadd(&rs, c);
 		if (c == '-') {
 		    prevpos = rsc.pos;
-		    ret.aux = TRUE;
+		    ret.aux = true;
 		    break;	       /* hyphen terminates word */
 		}
 	    }
@@ -519,7 +532,7 @@ token get_token(input *in) {
  * telling code paragraphs from paragraphs which merely start with
  * code).
  */
-int isbrace(input *in) {
+bool isbrace(input *in) {
     int c;
     filepos cpos;
 
@@ -567,6 +580,7 @@ static word *addword(word newword, word ***hptrptr) {
     if (!hptrptr)
 	return NULL;
     mnewword = snew(word);
+    newword.private_data = NULL;       /* placate gcc warning */
     *mnewword = newword;	       /* structure copy */
     mnewword->next = NULL;
     **hptrptr = mnewword;
@@ -599,13 +613,12 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		      tree234 *macros) {
     token t;
     paragraph par;
-    word wd, **whptr, **idximplicit;
+    word wd, **whptr, **idximplicit = NULL;
     wchar_t utext[2], *wdtext;
     int style, spcstyle;
-    int already;
-    int iswhite, seenwhite;
-    int type;
-    int prev_para_type;
+    bool already;
+    bool iswhite, seenwhite;
+    int prev_para_type = para_NotParaType;
     struct stack_item {
 	enum {
 	    stack_nop = 0,	       /* do nothing (for error recovery) */
@@ -623,19 +636,35 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
     stack parsestk;
     struct crossparaitem {
  	int type;		       /* currently c_lcont, c_quote or -1 */
-	int seen_lcont, seen_quote;
+	bool seen_lcont, seen_quote;
     };
     stack crossparastk;
-    word *indexword, *uword, *iword;
+    word *indexword = NULL, *uword, *iword;
     word *idxwordlist;
     rdstring indexstr;
-    int index_downcase, index_visible, indexing;
+    bool index_downcase = false, index_visible = false, indexing;
     const rdstring nullrs = { 0, 0, NULL };
     wchar_t uchr;
 
-    t.text = NULL;
-    t.origtext = NULL;
-    already = FALSE;
+    t = get_token(in);
+    already = true;
+
+    /*
+     * Ignore tok_white if it appears at the very start of the file.
+     *
+     * At the start of most paragraphs, tok_white is guaranteed not to
+     * appear, because get_token will have folded it into the
+     * preceding tok_eop (since a tok_eop is simply a sequence of
+     * whitespace containing at least two newlines).
+     *
+     * The one exception is if there isn't a preceding tok_eop, i.e.
+     * if the very first paragraph begins with something that lexes as
+     * a tok_white. Easiest way to get round that is to ignore it
+     * here, by unsetting the 'already' flag which will force a new
+     * token to be fetched below.
+     */
+    if (t.type == tok_white)
+        already = false;
 
     crossparastk = stk_new();
 
@@ -656,7 +685,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 	    if (!already) {
 		dtor(t), t = get_token(in);
 	    }
-	    already = FALSE;
+	    already = false;
 	} while (t.type == tok_eop);
 	if (t.type == tok_eof)
 	    break;
@@ -672,9 +701,10 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 	    while (1) {
 		dtor(t), t = get_codepar_token(in);
 		wd.type = wtype;
-		wd.breaks = FALSE;     /* shouldn't need this... */
+		wd.breaks = false;     /* shouldn't need this... */
 		wd.text = ustrdup(t.text);
 		wd.alt = NULL;
+                wd.aux = 0;
 		wd.fpos = t.pos;
 		addword(wd, &whptr);
 		dtor(t), t = get_token(in);
@@ -687,7 +717,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		if (t.type == tok_eop || t.type == tok_eof ||
 		    t.type == tok_rbrace) { /* might be } terminating \lcont */
 		    if (t.type == tok_rbrace)
-			already = TRUE;
+			already = true;
 		    break;
 		} else if (t.type == tok_cmd && t.cmd == c_c) {
 		    wtype = word_WeakCode;
@@ -698,7 +728,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			   wtype == word_WeakCode) {
 		    wtype = word_Strong;
 		} else {
-		    err_brokencodepara(&t.pos);
+		    err_brokencodepara(in->es, &t.pos);
 		    prev_para_type = par.type;
 		    addpara(par, ret);
 		    while (t.type != tok_eop)   /* error recovery: */
@@ -727,7 +757,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 	     */
 	    dtor(t), t = get_token(in);
 	    if (t.type != tok_lbrace) {
-		err_explbr(&t.pos);
+		err_explbr(in->es, &t.pos);
 		continue;
 	    }
 
@@ -739,7 +769,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 	    do {
 		dtor(t), t = get_token(in);
 	    } while (t.type == tok_white);
-	    already = TRUE;
+	    already = true;
 
 	    if (cmd == c_lcont) {
 		/*
@@ -750,16 +780,18 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		 */
 		sitem = snew(struct crossparaitem);
 		stop = (struct crossparaitem *)stk_top(crossparastk);
-		if (stop)
+		if (stop) {
 		    *sitem = *stop;
-		else
-		    sitem->seen_quote = sitem->seen_lcont = 0;
+                } else {
+		    sitem->seen_quote = false;
+                    sitem->seen_lcont = false;
+                }
 
 		if (prev_para_type == para_Bullet ||
 		    prev_para_type == para_NumberedList ||
 		    prev_para_type == para_Description) {
 		    sitem->type = c_lcont;
-		    sitem->seen_lcont = 1;
+		    sitem->seen_lcont = true;
 		    par.type = para_LcontPush;
 		    prev_para_type = par.type;
 		    addpara(par, ret);
@@ -770,7 +802,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		     * don't give a cascade error.
 		     */
 		    sitem->type = -1;
-		    err_misplacedlcont(&t.pos);
+		    err_misplacedlcont(in->es, &t.pos);
 		}
 	    } else {
 		/*
@@ -780,12 +812,14 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		 */
 		sitem = snew(struct crossparaitem);
 		stop = (struct crossparaitem *)stk_top(crossparastk);
-		if (stop)
+		if (stop) {
 		    *sitem = *stop;
-		else
-		    sitem->seen_quote = sitem->seen_lcont = 0;
+                } else {
+		    sitem->seen_quote = false;
+                    sitem->seen_lcont = false;
+                }
 		sitem->type = c_quote;
-		sitem->seen_quote = 1;
+		sitem->seen_quote = true;
 		par.type = para_QuotePush;
 		prev_para_type = par.type;
 		addpara(par, ret);
@@ -795,7 +829,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 	} else if (t.type == tok_rbrace) {
 	    struct crossparaitem *sitem = stk_pop(crossparastk);
 	    if (!sitem)
-		err_unexbrace(&t.pos);
+		err_unexbrace(in->es, &t.pos);
 	    else {
 		switch (sitem->type) {
 		  case c_lcont:
@@ -829,7 +863,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 	par.type = para_Normal;
 	if (t.type == tok_cmd) {
 	    int needkw;
-	    int is_macro = FALSE;
+	    bool is_macro = false;
 
 	    par.fpos = t.pos;
 	    switch (t.cmd) {
@@ -837,7 +871,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		needkw = -1;
 		break;
 	      case c__invalid:
-		err_badparatype(t.text, &t.pos);
+		err_badparatype(in->es, t.text, &t.pos);
 		needkw = 4;
 		break;
 	      case c__comment:
@@ -880,7 +914,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 	      case c_cfg: needkw = 8; par.type = para_Config;
 		start_cmd = c_cfg; break;
 	      case c_copyright: needkw = 32; par.type = para_Copyright; break;
-	      case c_define: is_macro = TRUE; needkw = 1; break;
+	      case c_define: is_macro = true; needkw = 1; break;
 		/* For \nocite the keyword is _everything_ */
 	      case c_nocite: needkw = 8; par.type = para_NoCite; break;
 	      case c_preamble: needkw = 32; par.type = para_Normal; break;
@@ -896,8 +930,9 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		par.type == para_UnnumberedChapter) {
 		struct crossparaitem *sitem = stk_top(crossparastk);
 		if (sitem && (sitem->seen_lcont || sitem->seen_quote)) {
-		    err_sectmarkerinblock(			  &t.pos,
-			  (sitem->seen_lcont ? "lcont" : "quote"));
+		    err_sectmarkerinblock(
+                        in->es, &t.pos,
+                        (sitem->seen_lcont ? "lcont" : "quote"));
 		}
 	    }
 
@@ -949,7 +984,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			}
 		    }
 		    if (t.type != tok_rbrace) {
-			err_kwunclosed(&t.pos);
+			err_kwunclosed(in->es, &t.pos);
 			continue;
 		    }
 		    rdadd(&rs, 0);     /* add string terminator */
@@ -962,11 +997,11 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 
 		/* See whether we have the right number of keywords. */
 		if ((needkw & 48) && nkeys > 0)
-		    err_kwillegal(&fp);
+		    err_kwillegal(in->es, &fp);
 		if ((needkw & 11) && nkeys == 0)
-		    err_kwexpected(&fp);
+		    err_kwexpected(in->es, &fp);
 		if ((needkw & 5) && nkeys > 1)
-		    err_kwtoomany(&fp);
+		    err_kwtoomany(in->es, &fp);
 
 		if (is_macro) {
 		    /*
@@ -985,7 +1020,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			if (t.type == tok_eop || t.type == tok_eof)
                             break;
 		    }
-		    macrodef(macros, rs.text, macrotext.text, fp);
+		    macrodef(macros, rs.text, macrotext.text, fp, in->es);
 		    continue;	       /* next paragraph */
 		}
 
@@ -1000,7 +1035,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		    if (t.type != tok_eop && t.type != tok_eof &&
 			(start_cmd == c__invalid ||
 			 t.type != tok_cmd || t.cmd != start_cmd)) {
-			err_bodyillegal(&t.pos);
+			err_bodyillegal(in->es, &t.pos);
 			/* Error recovery: eat the rest of the paragraph */
 			while (t.type != tok_eop && t.type != tok_eof &&
 			       (start_cmd == c__invalid ||
@@ -1008,7 +1043,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			    dtor(t), t = get_token(in);
 		    }
 		    if (t.type == tok_cmd)
-			already = TRUE;/* inhibit get_token at top of loop */
+			already = true;/* inhibit get_token at top of loop */
 		    prev_para_type = par.type;
 		    addpara(par, ret);
 
@@ -1040,16 +1075,16 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 	parsestk = stk_new();
 	style = word_Normal;
 	spcstyle = word_WhiteSpace;
-	indexing = FALSE;
-	seenwhite = TRUE;
+	indexing = false;
+	seenwhite = true;
 	while (t.type != tok_eop && t.type != tok_eof) {
-	    iswhite = FALSE;
-	    already = FALSE;
+	    iswhite = false;
+	    already = false;
 
 	    /* Handle implicit paragraph breaks after \IM, \BR etc */
 	    if (start_cmd != c__invalid &&
 		t.type == tok_cmd && t.cmd == start_cmd) {
-		already = TRUE;	       /* inhibit get_token at top of loop */
+		already = true;	       /* inhibit get_token at top of loop */
 		break;
 	    }
 
@@ -1077,7 +1112,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		wd.alt = NULL;
 		wd.aux = 0;
 		wd.fpos = t.pos;
-		wd.breaks = FALSE;
+		wd.breaks = false;
 
 		/*
 		 * Inhibit use of whitespace if it's (probably the
@@ -1086,7 +1121,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		 */
 		if (start_cmd != c__invalid) {
 		    dtor(t), t = get_token(in);
-		    already = TRUE;
+		    already = true;
 		    if (t.type == tok_cmd && t.cmd == start_cmd)
 			break;
 		}
@@ -1097,7 +1132,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		    addword(wd, &whptr);
 		if (indexing)
 		    addword(wd, &idximplicit);
-		iswhite = TRUE;
+		iswhite = true;
 		break;
 	      case tok_word:
 		if (indexing)
@@ -1117,7 +1152,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		}
 		break;
 	      case tok_lbrace:
-		err_unexbrace(&t.pos);
+		err_unexbrace(in->es, &t.pos);
 		/* Error recovery: push nop */
 		sitem = snew(struct stack_item);
 		sitem->type = stack_nop;
@@ -1133,7 +1168,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		     * wants popping. Accordingly, we treat it here
 		     * as an indication that the paragraph is over.
 		     */
-		    already = TRUE;
+		    already = true;
 		    goto finished_para;
 		} else {
 		    if (sitem->type & stack_ualt) {
@@ -1145,6 +1180,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			spcstyle = word_WhiteSpace;
 		    }
 		    if (sitem->type & stack_idx) {
+			rdadds(&indexstr, L"");
 			indexword->text = ustrdup(indexstr.text);
 			if (index_downcase) {
 			    word *w;
@@ -1156,10 +1192,10 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 				if (w->text)
 				    ustrlow(w->text);
 			}
-			indexing = FALSE;
+			indexing = false;
 			rdadd(&indexstr, L'\0');
-			index_merge(idx, FALSE, indexstr.text,
-				    idxwordlist, &sitem->fpos);
+			index_merge(idx, false, indexstr.text,
+				    idxwordlist, &sitem->fpos, in->es);
 			sfree(indexstr.text);
 		    }
 		    if (sitem->type & stack_hyper) {
@@ -1168,7 +1204,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			wd.alt = NULL;
 			wd.aux = 0;
 			wd.fpos = t.pos;
-			wd.breaks = FALSE;
+			wd.breaks = false;
 			if (!indexing || index_visible)
 			    addword(wd, &whptr);
 			if (indexing)
@@ -1180,7 +1216,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			wd.alt = NULL;
 			wd.aux = quote_Close;
 			wd.fpos = t.pos;
-			wd.breaks = FALSE;
+			wd.breaks = false;
 			if (!indexing || index_visible)
 			    addword(wd, &whptr);
 			if (indexing) {
@@ -1203,7 +1239,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		     */
 		    dtor(t), t = get_token(in);
 		    if (t.type != tok_lbrace) {
-			err_explbr(&t.pos);
+			err_explbr(in->es, &t.pos);
 		    } else {
 			int braces = 1;
 			while (braces > 0) {
@@ -1213,26 +1249,26 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			    else if (t.type == tok_rbrace)
 				braces--;
 			    else if (t.type == tok_eof) {
-				err_commenteof(&t.pos);
+				err_commenteof(in->es, &t.pos);
 				break;
 			    }
 			}
 		    }
 		    if (seenwhite) {
-			already = TRUE;
+			already = true;
 			dtor(t), t = get_token(in);
 			if (t.type == tok_white) {
-			    iswhite = TRUE;
-			    already = FALSE;
+			    iswhite = true;
+			    already = false;
 			}
 		    }
 		    break;
 		  case c_q:
-                  case c_cq:
-                    type = t.cmd;
+                  case c_cq: {
+                    int type = t.cmd;
 		    dtor(t), t = get_token(in);
 		    if (t.type != tok_lbrace) {
-			err_explbr(&t.pos);
+			err_explbr(in->es, &t.pos);
 		    } else {
 			/*
 			 * Enforce that \q may not be used anywhere
@@ -1252,7 +1288,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			    wd.alt = NULL;
 			    wd.aux = quote_Open;
 			    wd.fpos = t.pos;
-			    wd.breaks = FALSE;
+			    wd.breaks = false;
 			    if (!indexing || index_visible)
 				addword(wd, &whptr);
 			    if (indexing) {
@@ -1261,7 +1297,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			    }
 			    stype = stack_quote;
 			} else {
-			    err_codequote(&t.pos);
+			    err_codequote(in->es, &t.pos);
 			    stype = stack_nop;
 			}
 			sitem = snew(struct stack_item);
@@ -1269,7 +1305,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			sitem->type = stype;
                         if (type == c_cq) {
                             if (style != word_Normal) {
-                                err_nestedstyles(&t.pos);
+                                err_nestedstyles(in->es, &t.pos);
                             } else {
                                 style = word_WeakCode;
                                 spcstyle = tospacestyle(style);
@@ -1279,6 +1315,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			stk_push(parsestk, sitem);
 		    }
 		    break;
+                  }
 		  case c_K:
 		  case c_k:
 		  case c_W:
@@ -1289,7 +1326,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		     * brace. No nesting; no arguments.
 		     */
 		    wd.fpos = t.pos;
-		    wd.breaks = FALSE;
+		    wd.breaks = false;
 		    if (t.cmd == c_K)
 			wd.type = word_UpperXref;
 		    else if (t.cmd == c_k)
@@ -1301,13 +1338,13 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		    dtor(t), t = get_token(in);
 		    if (t.type != tok_lbrace) {
 			if (wd.type == word_Normal) {
-			    time_t thetime = time(NULL);
+			    time_t thetime = current_time();
 			    struct tm *broken = localtime(&thetime);
-			    already = TRUE;
+			    already = true;
 			    wdtext = ustrftime(NULL, broken);
 			    wd.type = style;
 			} else {
-			    err_explbr(&t.pos);
+			    err_explbr(in->es, &t.pos);
 			    wdtext = NULL;
 			}
 		    } else {
@@ -1320,7 +1357,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 				rdadds(&rs, t.text);
 			}
 			if (wd.type == word_Normal) {
-			    time_t thetime = time(NULL);
+			    time_t thetime = current_time();
 			    struct tm *broken = localtime(&thetime);
 			    wdtext = ustrftime(rs.text, broken);
 			    wd.type = style;
@@ -1329,7 +1366,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			}
 			sfree(rs.text);
 			if (t.type != tok_rbrace) {
-			    err_kwexprbr(&t.pos);
+			    err_kwexprbr(in->es, &t.pos);
 			}
 		    }
 		    wd.alt = NULL;
@@ -1359,7 +1396,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			if (t.type == tok_cmd &&
 			    (t.cmd == c_i || t.cmd == c_ii)) {
 			    if (indexing) {
-				err_nestedindex(&t.pos);
+				err_nestedindex(in->es, &t.pos);
 			    } else {
 				/* Add an index-reference word with no
 				 * text as yet */
@@ -1367,16 +1404,16 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 				wd.text = NULL;
 				wd.alt = NULL;
 				wd.aux = 0;
-				wd.breaks = FALSE;
+				wd.breaks = false;
 				indexword = addword(wd, &whptr);
 				/* Set up a rdstring to read the
 				 * index text */
 				indexstr = nullrs;
 				/* Flags so that we do the Right
 				 * Things with text */
-				index_visible = (type != c_I);
-				index_downcase = (type == c_ii);
-				indexing = TRUE;
+				index_visible = (t.cmd != c_I);
+				index_downcase = (t.cmd == c_ii);
+				indexing = true;
 				idxwordlist = NULL;
 				idximplicit = &idxwordlist;
 
@@ -1391,7 +1428,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			    (t.cmd == c_e || t.cmd == c_s ||
                              t.cmd == c_c || t.cmd == c_cw)) {
 			    if (style != word_Normal)
-				err_nestedstyles(&t.pos);
+				err_nestedstyles(in->es, &t.pos);
 			    else {
 				style = (t.cmd == c_c ? word_Code :
 					 t.cmd == c_cw ? word_WeakCode :
@@ -1403,7 +1440,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			    dtor(t), t = get_token(in);
 			}
 			if (t.type != tok_lbrace) {
-			    err_explbr(&t.pos);
+			    err_explbr(in->es, &t.pos);
 			    sfree(sitem);
 			} else {
 			    stk_push(parsestk, sitem);
@@ -1413,10 +1450,10 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		  case c_c:
 		  case c_cw:
 		  case c_e:
-		  case c_s:
-		    type = t.cmd;
+		  case c_s: {
+		    int type = t.cmd;
 		    if (style != word_Normal) {
-			err_nestedstyles(&t.pos);
+			err_nestedstyles(in->es, &t.pos);
 			/* Error recovery: eat lbrace, push nop. */
 			dtor(t), t = get_token(in);
 			sitem = snew(struct stack_item);
@@ -1426,7 +1463,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		    }
 		    dtor(t), t = get_token(in);
 		    if (t.type != tok_lbrace) {
-			err_explbr(&t.pos);
+			err_explbr(in->es, &t.pos);
 		    } else {
 			style = (type == c_c ? word_Code :
 				 type == c_cw ? word_WeakCode :
@@ -1439,12 +1476,13 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			stk_push(parsestk, sitem);
 		    }
 		    break;
+                  }
 		  case c_i:
 		  case c_ii:
-		  case c_I:
-		    type = t.cmd;
+		  case c_I: {
+		    int type = t.cmd;
 		    if (indexing) {
-			err_nestedindex(&t.pos);
+			err_nestedindex(in->es, &t.pos);
 			/* Error recovery: eat lbrace, push nop. */
 			dtor(t), t = get_token(in);
 			sitem = snew(struct stack_item);
@@ -1464,7 +1502,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 			(t.cmd == c_e || t.cmd == c_s ||
                          t.cmd == c_c || t.cmd == c_cw)) {
 			if (style != word_Normal)
-			    err_nestedstyles(&t.pos);
+			    err_nestedstyles(in->es, &t.pos);
 			else {
 			    style = (t.cmd == c_c ? word_Code :
 				     t.cmd == c_cw ? word_WeakCode :
@@ -1477,32 +1515,37 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		    }
 		    if (t.type != tok_lbrace) {
 			sfree(sitem);
-			err_explbr(&t.pos);
+			err_explbr(in->es, &t.pos);
 		    } else {
 			/* Add an index-reference word with no text as yet */
 			wd.type = word_IndexRef;
 			wd.text = NULL;
 			wd.alt = NULL;
 			wd.aux = 0;
-			wd.breaks = FALSE;
+			wd.breaks = false;
 			indexword = addword(wd, &whptr);
 			/* Set up a rdstring to read the index text */
 			indexstr = nullrs;
 			/* Flags so that we do the Right Things with text */
 			index_visible = (type != c_I);
 			index_downcase = (type == c_ii);
-			indexing = TRUE;
+			indexing = true;
 			idxwordlist = NULL;
 			idximplicit = &idxwordlist;
 			/* Stack item to close the indexing on exit */
 			stk_push(parsestk, sitem);
 		    }
 		    break;
+                  }
 		  case c_u:
 		    uchr = t.aux;
+                    if (uchr == 0) {
+                        err_zerochar(in->es, &t.pos);
+                        break;
+                    }
 		    utext[0] = uchr; utext[1] = 0;
 		    wd.type = style;
-		    wd.breaks = FALSE;
+		    wd.breaks = false;
 		    wd.alt = NULL;
 		    wd.aux = 0;
 		    wd.fpos = t.pos;
@@ -1535,12 +1578,12 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 		    } else {
 			if (indexing)
 			    rdadd(&indexstr, uchr);
-			already = TRUE;
+			already = true;
 		    }
 		    break;
 		  default:
 		    if (!macrolookup(macros, in, t.text, &t.pos))
-			err_badmidcmd(t.text, &t.pos);
+			err_badmidcmd(in->es, t.text, &t.pos);
 		    break;
 		}
 	    }
@@ -1553,7 +1596,7 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 	if (stk_top(parsestk)) {
 	    while ((sitem = stk_pop(parsestk)))
 		sfree(sitem);
-	    err_missingrbrace(&t.pos);
+	    err_missingrbrace(in->es, &t.pos);
 	}
 	stk_free(parsestk);
 	prev_para_type = par.type;
@@ -1569,13 +1612,13 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
 	    addpara(par, ret);
 	}
 	if (t.type == tok_eof)
-	    already = TRUE;
+	    already = true;
     }
 
     if (stk_top(crossparastk)) {
 	void *p;
 
-	err_missingrbrace2(&t.pos);
+	err_missingrbrace2(in->es, &t.pos);
 	while ((p = stk_pop(crossparastk)))
 	    sfree(p);
     }
@@ -1589,30 +1632,30 @@ static void read_file(paragraph ***ret, input *in, indexdata *idx,
     stk_free(crossparastk);
 }
 
-struct {
+const struct {
     char const *magic;
     size_t nmagic;
-    int binary;
-    void (*reader)(input *);
+    bool binary;
+    void (*reader)(input *, psdata *);
 } magics[] = {
-    { "%!FontType1-",     12, FALSE, &read_pfa_file },
-    { "%!PS-AdobeFont-",  15, FALSE, &read_pfa_file },
-    { "\x80\x01",          2, TRUE,  &read_pfb_file },
-    { "StartFontMetrics", 16, FALSE, &read_afm_file },
-    { "\x00\x01\x00\x00",  4, TRUE,  &read_sfnt_file },
-    { "true",		   4, TRUE,  &read_sfnt_file },
+    { "%!FontType1-",     12, false, &read_pfa_file },
+    { "%!PS-AdobeFont-",  15, false, &read_pfa_file },
+    { "\x80\x01",          2, true,  &read_pfb_file },
+    { "StartFontMetrics", 16, false, &read_afm_file },
+    { "\x00\x01\x00\x00",  4, true,  &read_sfnt_file },
+    { "true",		   4, true,  &read_sfnt_file },
 };
 
-paragraph *read_input(input *in, indexdata *idx) {
+paragraph *read_input(input *in, indexdata *idx, psdata *psd) {
     paragraph *head = NULL;
     paragraph **hptr = &head;
     tree234 *macros;
     char mag[16];
     size_t len, i;
-    int binary;
-    void (*reader)(input *);
+    bool binary;
+    void (*reader)(input *, psdata *);
 
-    macros = newtree234(macrocmp);
+    macros = newtree234(macrocmp, NULL);
 
     while (in->currindex < in->nfiles) {
 	setpos(in, in->filenames[in->currindex]);
@@ -1623,7 +1666,7 @@ paragraph *read_input(input *in, indexdata *idx) {
 
 	if (!in->filenames[in->currindex]) {
 	    in->currfp = stdin;
-	    in->wantclose = FALSE;     /* don't fclose stdin */
+	    in->wantclose = false;     /* don't fclose stdin */
 	    /*
 	     * When reading standard input, we always expect to see
 	     * an actual Halibut file and not any of the unusual
@@ -1637,10 +1680,10 @@ paragraph *read_input(input *in, indexdata *idx) {
 	     * looking at a text file type.
 	     */
 	    in->currfp = fopen(in->filenames[in->currindex], "rb");
-	    binary = FALSE; /* default to Halibut source, which is text */
+	    binary = false; /* default to Halibut source, which is text */
+            reader = NULL;
 	    if (in->currfp) {
-		in->wantclose = TRUE;
-		reader = NULL;
+		in->wantclose = true;
 		len = fread(mag, 1, sizeof(mag), in->currfp);
 		for (i = 0; i < lenof(magics); i++) {
 		    if (len >= magics[i].nmagic &&
@@ -1662,10 +1705,10 @@ paragraph *read_input(input *in, indexdata *idx) {
 	    if (reader == NULL) {
 		read_file(&hptr, in, idx, macros);
 	    } else {
-		(*reader)(in);
+		(*reader)(in, psd);
 	    }
 	} else {
-            err_cantopen(in->filenames[in->currindex]);
+            err_cantopen(in->es, in->filenames[in->currindex]);
         }
 	in->currindex++;
     }
