@@ -7,21 +7,24 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include "halibut.h"
+#include "paper.h"
 
 static void dbg_prtsource(paragraph *sourceform);
 static void dbg_prtwordlist(int level, word *w);
 static void dbg_prtkws(keywordlist *kws);
 
 static const struct pre_backend {
-    void *(*func)(paragraph *, keywordlist *, indexdata *);
+    void *(*func)(paragraph *, keywordlist *, indexdata *, psdata *,
+                  errorstate *);
     int bitfield;
 } pre_backends[] = {
     {paper_pre_backend, 0x0001}
 };
 
 static const struct backend {
-    char *name;
-    void (*func)(paragraph *, keywordlist *, indexdata *, void *);
+    const char *name;
+    void (*func)(paragraph *, keywordlist *, indexdata *, void *,
+                 errorstate *);
     paragraph *(*filename)(char *filename);
     int bitfield, prebackend_bitfield;
 } backends[] = {
@@ -41,16 +44,16 @@ static const struct backend {
 int main(int argc, char **argv) {
     char **infiles;
     int nfiles;
-    int nogo;
-    int errs;
-    int reportcols;
-    int list_fonts;
+    bool nogo;
+    bool reportcols;
+    bool list_fonts;
     int input_charset;
-    int debug;
+    bool debug;
     int backendbits, prebackbits;
     int k, b;
     paragraph *cfg, *cfg_tail;
     void *pre_backend_data[16];
+    errorstate es[1];
 
     /*
      * Use the specified locale everywhere. It'll be used for
@@ -68,13 +71,14 @@ int main(int argc, char **argv) {
      */
     infiles = snewn(argc, char *);
     nfiles = 0;
-    nogo = errs = FALSE;
-    reportcols = 0;
-    list_fonts = 0;
+    nogo = false;
+    reportcols = false;
+    list_fonts = false;
     input_charset = CS_ASCII;
-    debug = 0;
+    debug = false;
     backendbits = 0;
     cfg = cfg_tail = NULL;
+    es->fatal = false;
 
     if (argc == 1) {
 	usage();
@@ -129,34 +133,34 @@ int main(int argc, char **argv) {
 			    /* do nothing */;
 			} else if (!strcmp(opt, "-input-charset")) {
 			    if (!val) {
-				errs = TRUE, err_optnoarg(opt);
+				err_optnoarg(es, opt);
 			    } else {
 				int charset = charset_from_localenc(val);
 				if (charset == CS_NONE) {
-				    errs = TRUE, err_cmdcharset(val);
+				    err_cmdcharset(es, val);
 				} else {
 				    input_charset = charset;
 				}
 			    }
 			} else if (!strcmp(opt, "-help")) {
 			    help();
-			    nogo = TRUE;
+			    nogo = true;
 			} else if (!strcmp(opt, "-version")) {
 			    showversion();
-			    nogo = TRUE;
+			    nogo = true;
 			} else if (!strcmp(opt, "-licence") ||
 				   !strcmp(opt, "-license")) {
 			    licence();
-			    nogo = TRUE;
+			    nogo = true;
 			} else if (!strcmp(opt, "-list-charsets")) {
 			    listcharsets();
-			    nogo = TRUE;
+			    nogo = true;
 			} else if (!strcmp(opt, "-list-fonts")) {
-			    list_fonts = TRUE;
+			    list_fonts = true;
 			} else if (!strcmp(opt, "-precise")) {
-			    reportcols = 1;
+			    reportcols = true;
 			} else {
-			    errs = TRUE, err_nosuchopt(opt);
+			    err_nosuchopt(es, opt);
 			}
 		    }
 		    p = NULL;
@@ -172,21 +176,21 @@ int main(int argc, char **argv) {
 		    switch (c) {
 		      case 'h':
 			help();
-			nogo = TRUE;
+			nogo = true;
 			break;
 		      case 'V':
 			showversion();
-			nogo = TRUE;
+			nogo = true;
 			break;
 		      case 'L':
 			licence();
-			nogo = TRUE;
+			nogo = true;
 			break;
 		      case 'P':
-			reportcols = 1;
+			reportcols = true;
 			break;
 		      case 'd':
-			debug = TRUE;
+			debug = true;
 			break;
 		    }
 		    break;
@@ -201,7 +205,7 @@ int main(int argc, char **argv) {
 			char opt[2];
 			opt[0] = c;
 			opt[1] = '\0';
-			errs = TRUE, err_optnoarg(opt);
+			err_optnoarg(es, opt);
 		    }
 		    /*
 		     * Now c is the option and p is the parameter.
@@ -225,7 +229,7 @@ int main(int argc, char **argv) {
 				    *r = '\0';
 				    /* XXX ad-hoc diagnostic */
 				    if (!strcmp(s, "input-charset"))
-					err_futileopt("Cinput-charset",
+					err_futileopt(es, "Cinput-charset",
 					      "; use --input-charset");
 				    cmdline_cfg_add(para, s);
 				    r = s;
@@ -257,7 +261,7 @@ int main(int argc, char **argv) {
 			char opt[2];
 			opt[0] = c;
 			opt[1] = '\0';
-			errs = TRUE, err_nosuchopt(opt);
+			err_nosuchopt(es, opt);
 		    }
 		}
 	    }
@@ -272,7 +276,7 @@ int main(int argc, char **argv) {
 	}
     }
 
-    if (errs)
+    if (es->fatal)
 	exit(EXIT_FAILURE);
     if (nogo)
 	exit(EXIT_SUCCESS);
@@ -281,7 +285,7 @@ int main(int argc, char **argv) {
      * Do the work.
      */
     if (nfiles == 0 && !list_fonts) {
-	err_noinput();
+	err_noinput(es);
 	usage();
 	exit(EXIT_FAILURE);
     }
@@ -291,6 +295,7 @@ int main(int argc, char **argv) {
 	paragraph *sourceform, *p;
 	indexdata *idx;
 	keywordlist *keywords;
+        psdata *psd;
 
 	in.filenames = infiles;
 	in.nfiles = nfiles;
@@ -301,16 +306,19 @@ int main(int argc, char **argv) {
 	in.reportcols = reportcols;
 	in.stack = NULL;
 	in.defcharset = input_charset;
+        in.es = es;
 
 	idx = make_index();
+        psd = psdata_new();
 
-	sourceform = read_input(&in, idx);
+	sourceform = read_input(&in, idx, psd);
 	if (list_fonts) {
-	    listfonts();
+	    listfonts(psd);
 	    exit(EXIT_SUCCESS);
 	}
-	if (!sourceform)
+	if (es->fatal)
 	    exit(EXIT_FAILURE);
+        assert(sourceform);
 
 	/*
 	 * Append the config directives acquired from the command
@@ -331,15 +339,15 @@ int main(int argc, char **argv) {
 
 	sfree(infiles);
 
-	keywords = get_keywords(sourceform);
+	keywords = get_keywords(sourceform, es);
 	if (!keywords)
 	    exit(EXIT_FAILURE);
-	gen_citations(sourceform, keywords);
-	subst_keywords(sourceform, keywords);
+	gen_citations(sourceform, keywords, es);
+	subst_keywords(sourceform, keywords, es);
 
 	for (p = sourceform; p; p = p->next)
 	    if (p->type == para_IM)
-		index_merge(idx, TRUE, p->keyword, p->words, &p->fpos);
+		index_merge(idx, true, p->keyword, p->words, &p->fpos, es);
 
 	build_index(idx);
 
@@ -367,6 +375,7 @@ int main(int argc, char **argv) {
 	 * Select and run the pre-backends.
 	 */
 	prebackbits = 0;
+        memset(pre_backend_data, 0, sizeof(pre_backend_data));
 	for (k = 0; k < (int)lenof(backends); k++)
 	    if (backendbits == 0 || (backendbits & backends[k].bitfield))
 		prebackbits |= backends[k].prebackend_bitfield;
@@ -374,7 +383,7 @@ int main(int argc, char **argv) {
 	    if (prebackbits & pre_backends[k].bitfield) {
 		assert(k < (int)lenof(pre_backend_data));
 		pre_backend_data[k] =
-		    pre_backends[k].func(sourceform, keywords, idx);
+		    pre_backends[k].func(sourceform, keywords, idx, psd, es);
 	    }
 
 	/*
@@ -395,15 +404,19 @@ int main(int argc, char **argv) {
 			    break;
 			}
 			    
-		    backends[k].func(sourceform, keywords, idx, pbd);
+		    backends[k].func(sourceform, keywords, idx, pbd, es);
 		}
 	    }
 
 	free_para_list(sourceform);
 	free_keywords(keywords);
 	cleanup_index(idx);
+        psdata_free(psd);
     }
 
+    if (es->fatal)
+        exit(EXIT_FAILURE);
+
     return 0;
 }
 
