@@ -2,7 +2,10 @@
  * misc.c: miscellaneous useful items
  */
 
+#include <assert.h>
 #include <stdarg.h>
+#include <stdlib.h>
+#include <time.h>
 #include "halibut.h"
 
 char *adv(char *s) {
@@ -101,6 +104,19 @@ void rdaddsn(rdstringc *rs, char const *p, int len) {
     rs->pos += len;
     rs->text[rs->pos] = 0;
 }
+void rdaddc_rep(rdstringc *rs, char c, int len) {
+    if (len <= 0) {
+        assert(len == 0);
+        return;
+    }
+    if (rs->pos >= rs->size - len) {
+	rs->size = rs->pos + len + 128;
+	rs->text = sresize(rs->text, rs->size, char);
+    }
+    memset(rs->text + rs->pos, c, len);
+    rs->pos += len;
+    rs->text[rs->pos] = 0;
+}
 char *rdtrimc(rdstringc *rs) {
     rs->text = sresize(rs->text, rs->pos + 1, char);
     return rs->text;
@@ -237,7 +253,7 @@ void mark_attr_ends(word *words)
 
     wp = NULL;
     for (w = words; w; w = w->next) {
-	int both;
+	bool both;
 	if (!isvis(w->type))
 	    /* Invisible elements should not affect this calculation */
 	    continue;
@@ -580,3 +596,19 @@ paragraph *cmdline_cfg_simple(char *string, ...)
 
     return p;
 }
+
+/*
+ * Wrapper around the standard C time() function, which allows its
+ * return value to be overridden by the environment variable
+ * SOURCE_DATE_EPOCH, used to achieve reproducible builds by avoiding
+ * baking different datestamps into repetitions of what ought to be
+ * the same build.
+ */
+time_t current_time(void)
+{
+    const char *epoch = getenv("SOURCE_DATE_EPOCH");
+    if (epoch)
+        return atoll(epoch);
+
+    return time(NULL);
+}
