@@ -173,7 +173,7 @@ static int strcmp_chm(const char *a, const char *b)
     return 0;
 }
 
-int chm_directory_entry_cmp(void *av, void *bv)
+int chm_directory_entry_cmp(const void *av, const void *bv, void *cmpctx)
 {
     const struct chm_directory_entry
         *a = (const struct chm_directory_entry *)av,
@@ -181,7 +181,7 @@ int chm_directory_entry_cmp(void *av, void *bv)
     return strcmp_chm(a->filename, b->filename);
 }
 
-int chm_directory_entry_find(void *av, void *bv)
+int chm_directory_entry_find(const void *av, const void *bv, void *cmpctx)
 {
     const char *a = (const char *)av;
     const struct chm_directory_entry
@@ -235,7 +235,7 @@ static void directory(rdstringc *rs, tree234 *files)
     PUT_32BIT_LSB_FIRST(rs->text + dirhdr_size_field, rs->pos);
     PUT_32BIT_LSB_FIRST(rs->text + dirhdr_size2_field, rs->pos);
 
-    index = newtree234(NULL);
+    index = newtree234(NULL, NULL);
     curr_chunk = 0;
     depth = 1;
     /* Write out lowest-level PMGL chunks full of actual directory entries */
@@ -327,8 +327,7 @@ static void directory(rdstringc *rs, tree234 *files)
         PUT_32BIT_LSB_FIRST(chunk.text + chunk_endlen_field,
                             chunksize - chunk.pos);
         PUT_16BIT_LSB_FIRST(reversed_quickref.text, n_entries);
-        while (chunk.pos + reversed_quickref.pos < chunksize)
-            rdaddc(&chunk, 0);         /* zero-pad */
+        rdaddc_rep(&chunk, 0, chunksize - chunk.pos - reversed_quickref.pos);
         for (i = reversed_quickref.pos - 2; i >= 0; i -= 2)
             rdaddsn(&chunk, reversed_quickref.text+i, 2);
 
@@ -346,7 +345,7 @@ static void directory(rdstringc *rs, tree234 *files)
         int index_index = 0;
 
         prev_index = index;
-        index = newtree234(NULL);
+        index = newtree234(NULL, NULL);
         depth++;
 
         while (index_index < count234(prev_index)) {
@@ -417,8 +416,8 @@ static void directory(rdstringc *rs, tree234 *files)
             PUT_32BIT_LSB_FIRST(chunk.text + chunk_endlen_field,
                                 chunksize - chunk.pos);
             PUT_16BIT_LSB_FIRST(reversed_quickref.text, n_entries);
-            while (chunk.pos + reversed_quickref.pos < chunksize)
-                rdaddc(&chunk, 0);         /* zero-pad */
+            rdaddc_rep(&chunk, 0,
+                       chunksize - chunk.pos - reversed_quickref.pos);
             for (i = reversed_quickref.pos - 2; i >= 0; i -= 2)
                 rdaddsn(&chunk, reversed_quickref.text+i, 2);
 
@@ -510,7 +509,7 @@ struct chm_stringtab_entry {
     int strtab_offset;
 };
 
-static int chm_stringtab_cmp(void *av, void *bv)
+static int chm_stringtab_cmp(const void *av, const void *bv, void *cmpctx)
 {
     const struct chm_stringtab_entry
         *a = (const struct chm_stringtab_entry *)av,
@@ -519,7 +518,7 @@ static int chm_stringtab_cmp(void *av, void *bv)
                   b->chm->stringsfile.text + b->strtab_offset);
 }
 
-static int chm_stringtab_find(void *av, void *bv)
+static int chm_stringtab_find(const void *av, const void *bv, void *cmpctx)
 {
     const char *a = (const char *)av;
     const struct chm_stringtab_entry
@@ -535,16 +534,17 @@ int chm_intern_string(struct chm *chm, const char *string)
     if (!string)
         return 0;
 
-    if ((ent = (struct chm_stringtab_entry *)find234(
-             chm->stringtab, (void *)string, chm_stringtab_find)) == NULL) {
+    if ((ent = (struct chm_stringtab_entry *)findcmp234(
+             chm->stringtab, (void *)string, chm_stringtab_find, NULL)) ==
+        NULL) {
         ent = snew(struct chm_stringtab_entry);
         ent->chm = chm;
 
         /* Pad to ensure the string doesn't cross a page boundary. */
         size = strlen(string) + 1;  /* include the NUL terminator */
         assert(size < 0x1000);  /* avoid really serious trouble */
-        while ((chm->stringsfile.pos ^ (chm->stringsfile.pos + size-1)) >> 12)
-            rdaddc(&chm->stringsfile, 0);
+        if ((chm->stringsfile.pos ^ (chm->stringsfile.pos + size-1)) >> 12)
+            rdaddc_rep(&chm->stringsfile, 0, 0xFFF & -chm->stringsfile.pos);
 
         ent->strtab_offset = chm->stringsfile.pos;
         rdaddsc(&chm->stringsfile, string);
@@ -557,9 +557,9 @@ int chm_intern_string(struct chm *chm, const char *string)
 struct chm *chm_new(void)
 {
     struct chm *chm = snew(struct chm);
-    chm->files = newtree234(chm_directory_entry_cmp);
-    chm->windows = newtree234(NULL);
-    chm->stringtab = newtree234(chm_stringtab_cmp);
+    chm->files = newtree234(chm_directory_entry_cmp, NULL);
+    chm->windows = newtree234(NULL, NULL);
+    chm->stringtab = newtree234(chm_stringtab_cmp, NULL);
     chm->content0 = empty_rdstringc;
     chm->content1 = empty_rdstringc;
     chm->outfile = empty_rdstringc;
@@ -641,7 +641,8 @@ static void chm_add_file_internal(struct chm *chm, const char *name,
 static struct chm_directory_entry *chm_find_file(
     struct chm *chm, const char *name)
 {
-    return find234(chm->files, (void *)name, chm_directory_entry_find);
+    return findcmp234(chm->files, (const void *)name,
+                      chm_directory_entry_find, NULL);
 }
 
 static char *add_leading_slash(const char *str)
@@ -749,7 +750,7 @@ struct chm_urltbl_entry {
     int topics_offset_to_update;
 };
 
-int chm_urltbl_entry_cmp(void *av, void *bv)
+int chm_urltbl_entry_cmp(const void *av, const void *bv, void *cmpctx)
 {
     const struct chm_urltbl_entry
         *a = (const struct chm_urltbl_entry *)av,
@@ -895,7 +896,7 @@ const char *chm_build(struct chm *chm, int *outlen)
 
     {
         rdstringc winfile = {0, 0, NULL};
-        int i, j, s;
+        int i, s;
         struct chm_window *win;
 
         RDADD_32BIT_LSB_FIRST(&winfile, count234(chm->windows));
@@ -950,8 +951,7 @@ const char *chm_build(struct chm *chm, int *outlen)
             RDADD_32BIT_LSB_FIRST(&winfile, 0); /* default nav pane = TOC */
             RDADD_32BIT_LSB_FIRST(&winfile, 0); /* nav pane tabs at top */
             RDADD_32BIT_LSB_FIRST(&winfile, 0); /* WM_NOTIFY id */
-            for (j = 0; j < 20; j++)
-                rdaddc(&winfile, 0);            /* tab order block */
+            rdaddc_rep(&winfile, 0, 20);        /* tab order block */
             RDADD_32BIT_LSB_FIRST(&winfile, 0); /* history to keep */
             RDADD_32BIT_LSB_FIRST(&winfile, 0); /* no Jump 1 button target */
             RDADD_32BIT_LSB_FIRST(&winfile, 0); /* no Jump 2 button target */
@@ -976,15 +976,14 @@ const char *chm_build(struct chm *chm, int *outlen)
         rdstringc topics = {0, 0, NULL};
         rdstringc urltbl = {0, 0, NULL};
         rdstringc urlstr = {0, 0, NULL};
-        int i, index, s, n_tocidx_3;
+        int index, s, n_tocidx_3;
         struct chm_directory_entry *contentsfile = NULL, *indexfile = NULL;
         tree234 *urltbl_pre;
         struct chm_urltbl_entry *urltbl_entry;
 
-        urltbl_pre = newtree234(chm_urltbl_entry_cmp);
+        urltbl_pre = newtree234(chm_urltbl_entry_cmp, NULL);
 
-        for (i = 0; i < 0x1000; i++)
-            rdaddc(&tocidx, 0);
+        rdaddc_rep(&tocidx, 0, 0x1000);
 
         /* Write a header of one zero byte at the start of #URLSTR.
          * chmspec says this doesn't always appear, and is unclear on
@@ -1079,8 +1078,8 @@ const char *chm_build(struct chm *chm, int *outlen)
              * plus a NUL-terminated copy of the target file name / URL. */
             urlstr_size = 8 + strlen(sect->url) + 1;
             assert(urlstr_size < 0x1000); /* must _fit_ in a page! */
-            while ((urlstr.pos ^ (urlstr.pos + urlstr_size - 1)) >> 12)
-                rdaddc(&urlstr, 0);
+            if ((urlstr.pos ^ (urlstr.pos + urlstr_size - 1)) >> 12)
+                rdaddc_rep(&urlstr, 0, 0xFFF & -urlstr_size);
 
             /*
              * Save everything we know so far about the #URLTBL record
@@ -1229,8 +1228,7 @@ const char *chm_build(struct chm *chm, int *outlen)
         }
 
         /* Align the current #TOCIDX offset to 16 bytes */
-        while (tocidx.pos & 0xF)
-            rdaddc(&tocidx, 0);
+        rdaddc_rep(&tocidx, 0, 0xF & -tocidx.pos);
 
         /* #TOCIDX header field pointing at start of type-3 records */
         PUT_32BIT_LSB_FIRST(tocidx.text + 0x4, tocidx.pos);
@@ -1295,8 +1293,7 @@ const char *chm_build(struct chm *chm, int *outlen)
             RDADD_32BIT_LSB_FIRST(&sysfile, 1); /* unknown */
             RDADD_32BIT_LSB_FIRST(&sysfile, 0); /* no merge files */
             RDADD_32BIT_LSB_FIRST(&sysfile, 0); /* unknown */
-            while (sysfile.pos - idxhdr_start < 4096)
-                rdaddc(&sysfile, 0);
+            rdaddc_rep(&sysfile, 0, 4096 - (sysfile.pos - idxhdr_start));
 
             chm_add_file_internal(chm, "/#IDXHDR", sysfile.text + idxhdr_start,
                                   sysfile.pos - idxhdr_start,
@@ -1367,8 +1364,9 @@ const char *chm_build(struct chm *chm, int *outlen)
         int orig_decomp_size = chm->content1.pos;
         size_t i;
 
-        while (chm->content1.pos & 0x7FFF)
-            rdaddc(&chm->content1, 0); /* pad to a realign-interval boundary */
+        /* Pad to a realign-interval boundary */
+        rdaddc_rep(&chm->content1, 0, 0x7FFF & -chm->content1.pos);
+
         ef = lzx(chm->content1.text, chm->content1.pos, 0x8000, 0x10000);
         chm_add_file_internal(
             chm, "::DataSpace/Storage/MSCompressed/Content",
