@@ -215,7 +215,7 @@ static int whlp_file_offset(struct file *f);
 
 /* The master index maps file names to help-file offsets. */
 
-static int filecmp(void *av, void *bv)
+static int filecmp(const void *av, const void *bv, void *cmpctx)
 {
     const struct file *a = (const struct file *)av;
     const struct file *b = (const struct file *)bv;
@@ -241,7 +241,7 @@ static int fileleaf(const void *av, unsigned char *outbuf)
 
 /* The |CONTEXT internal file maps help context hashes to TOPICOFFSETs. */
 
-static int ctxcmp(void *av, void *bv)
+static int ctxcmp(const void *av, const void *bv, void *cmpctx)
 {
     const context *a = (const context *)av;
     const context *b = (const context *)bv;
@@ -269,7 +269,7 @@ static int ctxleaf(const void *av, unsigned char *outbuf)
 
 /* The |TTLBTREE internal file maps TOPICOFFSETs to title strings. */
 
-static int ttlcmp(void *av, void *bv)
+static int ttlcmp(const void *av, const void *bv, void *cmpctx)
 {
     const context *a = (const context *)av;
     const context *b = (const context *)bv;
@@ -299,7 +299,7 @@ static int ttlleaf(const void *av, unsigned char *outbuf)
 
 /* The |KWBTREE internal file maps index strings to TOPICOFFSETs. */
 
-static int idxcmp(void *av, void *bv)
+static int idxcmp(const void *av, const void *bv, void *cmpctx)
 {
     const struct indexrec *a = (const struct indexrec *)av;
     const struct indexrec *b = (const struct indexrec *)bv;
@@ -337,7 +337,7 @@ static int idxleaf(const void *av, unsigned char *outbuf)
  * is by the low 16 bits of the number (above that is flags).
  */
 
-static int tabcmp(void *av, void *bv)
+static int tabcmp(const void *av, const void *bv, void *cmpctx)
 {
     const int *a = (const int *)av;
     const int *b = (const int *)bv;
@@ -349,7 +349,7 @@ static int tabcmp(void *av, void *bv)
 }
 
 /* The internal `fontnames' B-tree stores strings. */
-static int fontcmp(void *av, void *bv)
+static int fontcmp(const void *av, const void *bv, void *cmpctx)
 {
     const char *a = (const char *)av;
     const char *b = (const char *)bv;
@@ -1241,7 +1241,7 @@ static void whlp_standard_systemsection(struct file *f)
     whlp_file_add_short(f, 0x36C);     /* magic number */
     whlp_file_add_short(f, 33);	       /* minor version: HCW 4.00 Win95+ */
     whlp_file_add_short(f, 1);	       /* major version */
-    whlp_file_add_long(f, time(NULL)); /* generation date */
+    whlp_file_add_long(f, current_time()); /* generation date */
     whlp_file_add_short(f, 0);	       /* flags=0 means no compression */
 
     /*
@@ -1328,8 +1328,11 @@ static void whlp_make_fontsection(WHLP h, struct file *f)
      */
     for (i = 0; (fontname = index234(h->fontnames, i)) != NULL; i++) {
 	char data[32];
-	memset(data, i, sizeof(data));
-	strncpy(data, fontname, sizeof(data));
+        size_t len = strlen(fontname);
+        if (len > sizeof(data))
+            len = sizeof(data);
+	memset(data, 0, sizeof(data));
+        memcpy(data, fontname, len);
 	whlp_file_add(f, data, sizeof(data));
     }
     
@@ -1340,7 +1343,7 @@ static void whlp_make_fontsection(WHLP h, struct file *f)
 	int fontpos;
 	void *ret;
 
-	ret = findpos234(h->fontnames, fontdesc->font, NULL, &fontpos);
+	ret = findpos234(h->fontnames, fontdesc->font, &fontpos);
 	assert(ret != NULL);
 
 	whlp_file_add_char(f, fontdesc->rendition);
@@ -1711,15 +1714,15 @@ WHLP whlp_new(void)
     /*
      * Internal B-trees.
      */
-    ret->files = newtree234(filecmp);
-    ret->pre_contexts = newtree234(NULL);
-    ret->contexts = newtree234(ctxcmp);
-    ret->titles = newtree234(ttlcmp);
-    ret->text = newtree234(NULL);
-    ret->index = newtree234(idxcmp);
-    ret->tabstops = newtree234(tabcmp);
-    ret->fontnames = newtree234(fontcmp);
-    ret->fontdescs = newtree234(NULL);
+    ret->files = newtree234(filecmp, NULL);
+    ret->pre_contexts = newtree234(NULL, NULL);
+    ret->contexts = newtree234(ctxcmp, NULL);
+    ret->titles = newtree234(ttlcmp, NULL);
+    ret->text = newtree234(NULL, NULL);
+    ret->index = newtree234(idxcmp, NULL);
+    ret->tabstops = newtree234(tabcmp, NULL);
+    ret->fontnames = newtree234(fontcmp, NULL);
+    ret->fontdescs = newtree234(NULL, NULL);
 
     /*
      * Some standard files.
@@ -1747,7 +1750,7 @@ void whlp_close(WHLP h, char *filename)
     int filecount, offset, index, filelen;
     struct file *file, *map, *md;
     context *ctx;
-    int has_index;
+    bool has_index;
 
     /*
      * Lay out the topic section.
